@@ -18,125 +18,17 @@
 
 #include <string.h>
 #include "7zip.h"
+#include "gzip_et_al.h"
 #include "interface.h"
 #include "main.h"
 #include "string_utils.h"
 #include "support.h"
 #include "window.h"
 
+#define INDEX (archive->type == XARCHIVETYPE_RAR ? (archive->version == 5 ? 1 : 0) : 0)
+
 static gboolean data_line, encrypted, last_line;
 
-void xa_7zip_ask (XArchive *archive)
-{
-	archive->can_test = TRUE;
-	archive->can_extract = TRUE;
-	archive->can_add = TRUE;
-	archive->can_delete = TRUE;
-	archive->can_sfx = TRUE;
-	archive->can_password = TRUE;
-	archive->can_full_path[0] = TRUE;
-	archive->can_overwrite = TRUE;
-	archive->can_update[1] = TRUE;
-	archive->can_solid = TRUE;
-}
-
-static gchar *xa_7zip_password_str (XArchive *archive)
-{
-	if (archive->password)
-		return g_strconcat(" -p", archive->password, NULL);
-	else
-		return g_strdup("");
-}
-
-static void xa_7zip_parse_output (gchar *line, XArchive *archive)
-{
-	XEntry *entry;
-	gchar *filename;
-	gpointer item[5];
-	gint linesize = 0,a = 0;
-	gboolean dir;
-
-	if (last_line)
-		return;
-
-	if (!data_line)
-	{
-		if (strncmp(line, "Method = ", 9) == 0 && strstr(line, "7zAES"))
-		{
-			encrypted = TRUE;
-			archive->has_password = TRUE;
-		}
-
-		if ((line[0] == '-') && line[3])
-		{
-			data_line = TRUE;
-			return;
-		}
-		return;
-	}
-	if (line[0] == '-')
-	{
-		last_line = TRUE;
-		return;
-	}
-
-	linesize = strlen(line);
-	archive->files++;
-
-	/* Date */
-	line[10] = '\0';
-	item[4] = line;
-
-	/* Time */
-	line[19] = '\0';
-	item[3] = line + 11;
-
-	/* Permissions */
-	line[25] = '\0';
-	item[2] = line + 20;
-
-	dir = (*(char *) item[2] == 'D');
-
-	/* Size */
-	for(a=26; a < linesize; ++a)
-		if(line[a] >= '0' && line[a] <= '9')
-			break;
-
-	line[38] = '\0';
-	item[0] = line + a;
-	archive->files_size += g_ascii_strtoull(item[0],NULL,0);
-
-	/* Compressed */
-	/* Is this item solid? */
-	if (line[50] == ' ')
-	{
-		line[linesize-1] = '\0';
-		item[1] = "0";
-	}
-	else
-	{
-		for(a=39; a < linesize; ++a)
-			if(line[a] >= '0' && line[a] <= '9')
-				break;
-		line[51] = '\0';
-		item[1] = line + a;
-		line[linesize-1] = '\0';
-	}
-
-	filename = g_strdup(line + 53);
-	entry = xa_set_archive_entries_for_each_row (archive,filename,item);
-
-	if (entry != NULL)
-	{
-		if (dir)
-			entry->is_dir = TRUE;
-
-		entry->is_encrypted = encrypted;
-	}
-
-	g_free(filename);
-}
-
 static void xa_7zip_uint64_skip (GIOChannel *file)
 {
 	gchar first, byte;
@@ -152,21 +44,18 @@ static void xa_7zip_uint64_skip (GIOChannel *file)
 	}
 }
 
-void xa_7zip_list (XArchive *archive)
+gboolean is7zip_mhe (const gchar *filename)
 {
-	const GType types[] = {GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_UINT64, G_TYPE_UINT64, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER};
-	const gchar *titles[] = {_("Original"), _("Compressed"), _("Attr"), _("Time"), _("Date")};
 	GIOChannel *file;
-	gchar *password_str, *command;
 	guint i;
+	gchar byte;
+	guint64 offset = 0;
+	gboolean result = FALSE;
 
-	file = g_io_channel_new_file(archive->path[0], "r", NULL);
+	file = g_io_channel_new_file(filename, "r", NULL);
 
 	if (file)
 	{
-		gchar byte;
-		guint64 offset = 0;
-
 		g_io_channel_set_encoding(file, NULL, NULL);
 
 		/* skip signature, version and header CRC32 */
@@ -176,7 +65,7 @@ void xa_7zip_list (XArchive *archive)
 		for (i = 0; i < 8; i++)
 		{
 			g_io_channel_read_chars(file, &byte, sizeof(byte), NULL, NULL);
-			offset |= (guint64) byte << (8 * i);
+			offset |= (guint64) (guchar) byte << (8 * i);
 		}
 
 		/* skip next header size and CRC32 */
@@ -247,7 +136,7 @@ void xa_7zip_list (XArchive *archive)
 											g_io_channel_read_chars(file, id, sizeof(id), NULL, NULL);
 
 											/* check for id of 7zAES */
-											archive->has_password = (memcmp(id, "\x06\xf1\x07\x01", 4) == 0);
+											result = (memcmp(id, "\x06\xf1\x07\x01", 4) == 0);
 										}
 									}
 								}
@@ -259,18 +148,156 @@ void xa_7zip_list (XArchive *archive)
 		}
 
 		g_io_channel_shutdown(file, FALSE, NULL);
-
-		if (archive->has_password)
-			if (!xa_check_password(archive))
-				return;
 	}
 
+	return result;
+}
+
+/* it can handle other archive types as well */
+void xa_7zip_ask (XArchive *archive)
+{
+	archive->can_test = TRUE;
+	archive->can_extract = TRUE;
+	archive->can_add = archiver[archive->type].is_compressor;
+	archive->can_delete = (archiver[archive->type].is_compressor && !SINGLE_FILE_COMPRESSOR(archive));
+	archive->can_sfx = (archive->type == XARCHIVETYPE_7ZIP);
+	archive->can_password = (archive->type == XARCHIVETYPE_7ZIP);
+	archive->can_full_path[0] = TRUE;
+	archive->can_overwrite = TRUE;
+	archive->can_update[1] = archiver[archive->type].is_compressor;
+	archive->can_freshen[1] = (archiver[archive->type].is_compressor && !SINGLE_FILE_COMPRESSOR(archive));
+	archive->can_move = archiver[archive->type].is_compressor;
+	archive->can_solid = (archive->type == XARCHIVETYPE_7ZIP);
+}
+
+static gchar *xa_7zip_password_str (XArchive *archive)
+{
+	if (archive->password)
+		return g_strconcat(" -p", archive->password, NULL);
+	else
+		return g_strdup("");
+}
+
+static void xa_7zip_parse_output (gchar *line, XArchive *archive)
+{
+	XEntry *entry;
+	gchar *filename;
+	gpointer item[5];
+	gint linesize = 0,a = 0;
+	gboolean dir;
+
+	if (last_line)
+		return;
+
+	if (!data_line)
+	{
+		if (strncmp(line, "Method = ", 9) == 0 && strstr(line, "7zAES"))
+		{
+			encrypted = TRUE;
+			archive->has_password = TRUE;
+		}
+
+		if ((line[0] == '-') && line[3])
+		{
+			data_line = TRUE;
+			return;
+		}
+		return;
+	}
+	if (line[0] == '-')
+	{
+		last_line = TRUE;
+		return;
+	}
+
+	linesize = strlen(line);
+
+	/* Date */
+	line[10] = '\0';
+	item[2] = line;
+
+	/* Time */
+	line[19] = '\0';
+	item[3] = line + 11;
+
+	/* Permissions */
+	line[25] = '\0';
+	item[4] = line + 20;
+
+	dir = (*(char *) item[4] == 'D');
+
+	/* Size */
+	for(a=26; a < linesize; ++a)
+		if(line[a] >= '0' && line[a] <= '9')
+			break;
+
+	line[38] = '\0';
+	item[0] = line + a;
+
+	/* Compressed */
+	/* Is this item solid? */
+	if (line[50] == ' ')
+	{
+		line[linesize-1] = '\0';
+		item[1] = "0";
+	}
+	else
+	{
+		for(a=39; a < linesize; ++a)
+			if(line[a] >= '0' && line[a] <= '9')
+				break;
+		line[51] = '\0';
+		item[1] = line + a;
+		line[linesize-1] = '\0';
+	}
+
+	filename = g_strdup(line + 53);
+	entry = xa_set_archive_entries_for_each_row(archive, filename, item);
+
+	if (entry)
+	{
+		if (dir)
+			entry->is_dir = TRUE;
+
+		entry->is_encrypted = encrypted;
+
+		if (!entry->is_dir)
+			archive->files++;
+
+		archive->files_size += g_ascii_strtoull(item[0], NULL, 0);
+	}
+
+	g_free(filename);
+}
+
+void xa_7zip_list (XArchive *archive)
+{
+	const GType types[] = {GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_UINT64, G_TYPE_UINT64, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER};
+	const gchar *titles[] = {_("Original Size"), _("Compressed"), _("Date"), _("Time"), _("Attributes")};
+	gchar *password_str, *command;
+	guint i;
+
+	if (!archive->has_password)
+		archive->has_password = is7zip_mhe(archive->path[0]);
+
+	if (archive->has_password)
+		if (!xa_check_password(archive))
+			return;
+
+	/* a single file compressor archive is no longer new and empty now */
+	archive->can_add = (archiver[archive->type].is_compressor && !SINGLE_FILE_COMPRESSOR(archive));
+
 	data_line = FALSE;
 	last_line = FALSE;
 	encrypted = FALSE;
 
 	password_str = xa_7zip_password_str(archive);
-	command = g_strconcat(archiver[archive->type].program[0], " l", password_str, " ", archive->path[1], NULL);
+
+	if (archive->type == XARCHIVETYPE_CPIO && archive->version == 'E')
+		command = g_strconcat("sh -c \"echo ", _("Unsupported binary format!"), " >&2; exit 1\"", NULL);
+	else
+		command = g_strconcat(archiver[archive->type].program[INDEX], " l", password_str, " ", archive->path[1], NULL);
+
 	g_free(password_str);
 
 	archive->files_size = 0;
@@ -280,6 +307,7 @@ void xa_7zip_list (XArchive *archive)
 	g_free ( command );
 
 	archive->columns = 8;
+	archive->size_column = 2;
 	archive->column_types = g_malloc0(sizeof(types));
 
 	for (i = 0; i < archive->columns; i++)
@@ -293,7 +321,7 @@ void xa_7zip_test (XArchive *archive)
 	gchar *password_str, *command;
 
 	password_str = xa_7zip_password_str(archive);
-	command = g_strconcat(archiver[archive->type].program[0], " t", password_str, " -bd -y ", archive->path[1], NULL);
+	command = g_strconcat(archiver[archive->type].program[INDEX], " t", password_str, " -bd -y ", archive->path[1], NULL);
 	g_free(password_str);
 
 	xa_run_command(archive, command);
@@ -318,7 +346,7 @@ gboolean xa_7zip_extract (XArchive *archive, GSList *file_list)
 
 	files = xa_quote_filenames(file_list, NULL, TRUE);
 	password_str = xa_7zip_password_str(archive);
-	command = g_strconcat(archiver[archive->type].program[0],
+	command = g_strconcat(archiver[archive->type].program[INDEX],
 	                      archive->do_full_path ? " x" : " e",
 	                      archive->do_overwrite ? " -aoa" : " -aos",
 	                      password_str, " -bd -spd -y ",
@@ -336,7 +364,7 @@ gboolean xa_7zip_extract (XArchive *archive, GSList *file_list)
 void xa_7zip_add (XArchive *archive, GSList *file_list, gchar *compression)
 {
 	GString *files;
-	gchar *password_str, *command;
+	gchar *password_str, *solid, *command;
 
 	if (archive->location_path != NULL)
 		archive->child_dir = g_strdup(archive->working_dir);
@@ -346,12 +374,16 @@ void xa_7zip_add (XArchive *archive, GSList *file_list, gchar *compression)
 
 	files = xa_quote_filenames(file_list, NULL, TRUE);
 	password_str = xa_7zip_password_str(archive);
+	solid = g_strconcat(" -ms=", archive->do_solid ? "on" : "off", NULL);
 	command = g_strconcat(archiver[archive->type].program[0],
 	                      archive->do_update ? " u" : " a",
-	                      " -ms=", archive->do_solid ? "on" : "off",
+	                      archive->do_freshen ? " -ur0w0x1z1" : "",
+	                      archive->do_move ? " -sdel" : "",
+	                      archive->type == XARCHIVETYPE_7ZIP ? solid : "",
 	                      " -mx=", compression,
 	                      password_str, " -bd -spd -y ",
 	                      archive->path[1], files->str, NULL);
+	g_free(solid);
 	g_free(password_str);
 	g_string_free(files,TRUE);
 
