@@ -91,7 +91,20 @@ static gboolean xa_process_stderr (GIOChannel *ioc, GIOCondition cond, XArchive
 		if (status == G_IO_STATUS_NORMAL)
 		{
 			if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_window->store_output)))
+			{
+				if (!g_utf8_validate(line, -1, NULL))
+				{
+					gchar *utf8 = g_locale_to_utf8(line, -1, NULL, NULL, NULL);
+
+					if (utf8)
+					{
+						g_free(line);
+						line = utf8;
+					}
+				}
+
 				archive->output = g_slist_prepend(archive->output, g_strdup(line));
+			}
 
 			g_free(line);
 		}
@@ -164,7 +177,7 @@ static XEntry *xa_alloc_memory_for_each_row (guint columns, GType column_types[]
 	return entry;
 }
 
-static XEntry *xa_find_child_entry (XEntry *entry, gchar *string)
+static XEntry *xa_find_directory_entry (XEntry *entry, const gchar *name)
 {
 	gchar *filename;
 
@@ -172,9 +185,9 @@ static XEntry *xa_find_child_entry (XEntry *entry, gchar *string)
 		return NULL;
 
 	if (g_utf8_validate(entry->filename, -1, NULL))
-		filename = g_filename_display_name(string);
+		filename = g_filename_display_name(name);
 	else
-		filename = g_strdup(string);
+		filename = g_strdup(name);
 
 	if (entry->is_dir && strcmp(entry->filename, filename) == 0)
 	{
@@ -184,7 +197,7 @@ static XEntry *xa_find_child_entry (XEntry *entry, gchar *string)
 
 	g_free(filename);
 
-  return xa_find_child_entry(entry->next, string);
+  return xa_find_directory_entry(entry->next, name);
 }
 
 static gpointer *xa_fill_archive_entry_columns_for_each_row (XArchive *archive, XEntry *entry, gpointer *items)
@@ -266,25 +279,82 @@ static gboolean _xa_dir_sidebar_select_row (GtkTreeModel *model, GtkTreePath *pa
 	return value;
 }
 
-XArchive *xa_init_archive_structure(gint type)
+gboolean xa_get_compressed_tar_type (XArchiveType *type)
 {
-	XEntry *entry = NULL;
-	XArchive *archive = NULL;
+	switch (*type)
+	{
+		case XARCHIVETYPE_BZIP2:
+			*type = XARCHIVETYPE_TAR_BZ2;
+			break;
 
-	archive = g_new0(XArchive,1);
-	if (archive == NULL)
+		case XARCHIVETYPE_COMPRESS:
+			*type = XARCHIVETYPE_TAR_Z;
+			break;
+
+		case XARCHIVETYPE_GZIP:
+			*type = XARCHIVETYPE_TAR_GZ;
+			break;
+
+		case XARCHIVETYPE_LZ4:
+			*type = XARCHIVETYPE_TAR_LZ4;
+			break;
+
+		case XARCHIVETYPE_LZIP:
+			*type = XARCHIVETYPE_TAR_LZ;
+			break;
+
+		case XARCHIVETYPE_LZMA:
+			*type = XARCHIVETYPE_TAR_LZMA;
+			break;
+
+		case XARCHIVETYPE_LZOP:
+			*type = XARCHIVETYPE_TAR_LZOP;
+			break;
+
+		case XARCHIVETYPE_XZ:
+			*type = XARCHIVETYPE_TAR_XZ;
+			break;
+
+		default:
+			return FALSE;
+	}
+
+	return TRUE;
+}
+
+XArchive *xa_init_archive_structure (ArchiveType xa)
+{
+	XArchive *archive;
+	XEntry *entry;
+
+	archive = g_new0(XArchive, 1);
+
+	if (!archive)
 		return NULL;
-	entry = g_new0(XEntry,1);
+
+	archive->type = xa.type;
+	archive->version = xa.version;
+
+	entry = g_new0(XEntry, 1);
+
+	if (!entry)
+	{
+		g_free(archive);
+		return NULL;
+	}
+
 	entry->filename = "";
 	archive->root_entry = entry;
-	archive->ask =		ask[type];
-	archive->list =		list[type];
-	archive->delete =	delete[type];
-	archive->add =		add[type];
-	archive->extract = 	extract[type];
-	archive->test = 	test[type];
-	archive->type = type;
+
+	archive->ask = ask[xa.type];
+	archive->list = list[xa.type];
+	archive->test = test[xa.type];
+	archive->extract = extract[xa.type];
+	archive->add = add[xa.type];
+	archive->delete = delete[xa.type];
+
 	(*archive->ask)(archive);
+
 	return archive;
 }
 
@@ -346,7 +416,7 @@ void xa_spawn_async_process (XArchive *archive, const gchar *command)
 	g_io_add_watch(ioc, G_IO_IN | G_IO_ERR | G_IO_HUP | G_IO_NVAL, (GIOFunc) xa_process_stdout, archive);
 
 	ioc = g_io_channel_unix_new(archive->child_fderr);
-	g_io_channel_set_encoding(ioc, locale, NULL);
+	g_io_channel_set_encoding(ioc, NULL, NULL);
 	g_io_channel_set_flags(ioc, G_IO_FLAG_NONBLOCK, NULL);
 	g_io_add_watch(ioc, G_IO_IN | G_IO_ERR | G_IO_HUP | G_IO_NVAL, (GIOFunc) xa_process_stderr, archive);
 
@@ -536,67 +606,79 @@ void xa_free_entry (XArchive *archive, XEntry *entry)
 	g_free(entry);
 }
 
-XEntry *xa_set_archive_entries_for_each_row (XArchive *archive,gchar *filename,gpointer *items)
+XEntry *xa_set_archive_entries_for_each_row (XArchive *archive, const gchar *filename, gpointer *items)
 {
-	XEntry *new_entry= NULL;
-	XEntry *last_entry = archive->root_entry;
-	gchar **components = NULL;
-	unsigned short int x = 0;
+	XEntry *entry = NULL, *last = archive->root_entry;
+	gchar **components;
+	guint n = 0;
 
 	/* check for batch mode */
 	if (!xa_main_window)
 		return NULL;
 
-	components = g_strsplit(filename,"/",-1);
+	components = g_strsplit(filename, "/", -1);
 
 	if (*filename == '/')
 	{
 		gchar *slashdir;
 
-		x = 1;
-		slashdir = g_strconcat("/", components[x], NULL);
-		g_free(components[x]);
-		components[x] = slashdir;
+		n = 1;
+		slashdir = g_strconcat("/", components[n], NULL);
+		g_free(components[n]);
+		components[n] = slashdir;
 	}
 
-	while (components[x] && strlen(components[x]) > 0)
+	while (components[n] && *components[n])
 	{
-		new_entry = xa_find_child_entry(last_entry->child,components[x]);
-		if (new_entry == NULL)
+		entry = xa_find_directory_entry(last->child, components[n]);
+
+		if (entry == NULL)
 		{
-			new_entry = xa_alloc_memory_for_each_row(archive->columns, archive->column_types);
-			if (new_entry == NULL)
+			entry = xa_alloc_memory_for_each_row(archive->columns, archive->column_types);
+
+			if (entry == NULL)
 				return NULL;
-			new_entry->filename = g_strdup(components[x]);
-			new_entry->columns = xa_fill_archive_entry_columns_for_each_row(archive,new_entry,items);
-			if (components[x+1] != NULL)
-				new_entry->is_dir = TRUE;
-			new_entry->next = last_entry->child;
-			last_entry->child = new_entry;
-			new_entry->prev = last_entry;
+
+			entry->filename = g_strdup(components[n]);
+
+			if (components[n + 1])
+				entry->is_dir = TRUE;
+
+			entry->next = last->child;
+			last->child = entry;
+			entry->prev = last;
 		}
-		last_entry = new_entry;
-		x++;
+
+		if (components[n + 1] == NULL || *components[n + 1] == 0)
+			entry->columns = xa_fill_archive_entry_columns_for_each_row(archive, entry, items);
+
+		last = entry;
+		n++;
 	}
+
 	g_strfreev(components);
-	return new_entry;
+
+	return entry;
 }
 
-XEntry* xa_find_entry_from_path (XEntry *root_entry,const gchar *fullpathname)
+XEntry* xa_find_entry_from_dirpath (XArchive *archive, const gchar *dirpath)
 {
-	gchar **components = NULL;
-	unsigned short int x = 0;
-	XEntry *new_entry = NULL;
+	XEntry *root = archive->root_entry, *entry = NULL;
+	gchar **components;
+	guint n = 0;
 
-	components = g_strsplit(fullpathname,"/",-1);
-	while (components[x] && strlen(components[x]) > 0)
+	components = g_strsplit(dirpath, "/", -1);
+
+	while (components[n] && *components[n])
 	{
-		new_entry = xa_find_child_entry(root_entry->child,components[x]);
-		root_entry = new_entry;
-		x++;
+		entry = xa_find_directory_entry(root->child, components[n]);
+		root = entry;
+		n++;
 	}
+
 	g_strfreev(components);
-	return new_entry;
+
+	return entry;
 }
 
 gchar *xa_build_full_path_name_from_entry (XEntry *entry)
