@@ -26,9 +26,13 @@ typedef enum
 	XARCHIVETYPE_7ZIP = XARCHIVETYPE_FIRST,
 	XARCHIVETYPE_ARJ,
 	XARCHIVETYPE_BZIP2,
+	XARCHIVETYPE_COMPRESS,
+	XARCHIVETYPE_CPIO,
 	XARCHIVETYPE_DEB,
 	XARCHIVETYPE_GZIP,
 	XARCHIVETYPE_LHA,
+	XARCHIVETYPE_LZ4,
+	XARCHIVETYPE_LZIP,
 	XARCHIVETYPE_LZMA,
 	XARCHIVETYPE_LZOP,
 	XARCHIVETYPE_RAR,
@@ -36,9 +40,12 @@ typedef enum
 	XARCHIVETYPE_TAR,
 	XARCHIVETYPE_TAR_BZ2,
 	XARCHIVETYPE_TAR_GZ,
+	XARCHIVETYPE_TAR_LZ,
+	XARCHIVETYPE_TAR_LZ4,
 	XARCHIVETYPE_TAR_LZMA,
 	XARCHIVETYPE_TAR_LZOP,
 	XARCHIVETYPE_TAR_XZ,
+	XARCHIVETYPE_TAR_Z,
 	XARCHIVETYPE_XZ,
 	XARCHIVETYPE_ZIP,
 	XARCHIVETYPE_TYPES,
@@ -73,8 +80,15 @@ typedef struct
 	gboolean is_compressor;
 	GSList *type;
 	GSList *glob;
+	GSList *version;
 } XArchiver;
 
+typedef struct
+{
+	XArchiveType type;
+	gushort version;
+} ArchiveType;
+
 typedef struct _XEntry XEntry;
 
 struct _XEntry
@@ -115,6 +129,7 @@ struct _XArchive
 	GtkTreeModel *model;
 	GtkListStore *liststore;
 	guint columns;
+	guint size_column;
 	GType *column_types;
 	GtkCellRenderer *pixbuf_renderer;
 	GtkCellRenderer *text_renderer;
@@ -201,12 +216,13 @@ void xa_detect_encrypted_archive(XArchive *);
 void xa_fill_dir_sidebar(XArchive *, gboolean);
 void xa_fill_list_with_recursed_entries(XEntry *, GSList **);
 gint xa_find_archive_index(gint);
-XEntry *xa_find_entry_from_path(XEntry *, const gchar *);
+XEntry *xa_find_entry_from_dirpath(XArchive *, const gchar *);
 void xa_free_entry(XArchive *, XEntry *);
+gboolean xa_get_compressed_tar_type(XArchiveType *);
 gint xa_get_new_archive_idx();
-XArchive *xa_init_archive_structure(gint);
+XArchive *xa_init_archive_structure(ArchiveType);
 gboolean xa_run_command(XArchive *, const gchar *);
-XEntry *xa_set_archive_entries_for_each_row(XArchive *, gchar *, gpointer *);
+XEntry *xa_set_archive_entries_for_each_row(XArchive *, const gchar *, gpointer *);
 void xa_dir_sidebar_row_selected(GtkTreeSelection *, gpointer);
 void xa_dir_sidebar_select_row(XEntry *);
 gint xa_sort_dirs_before_files(GtkTreeModel *, GtkTreeIter *, GtkTreeIter *, gpointer);
