@@ -86,12 +86,16 @@ static void xa_multi_extract_dialog_selection_changed (GtkTreeSelection *selecti
 {
 	GtkTreeIter iter;
 	GtkTreeModel *model;
-	gint type;
+	XArchiveType type;
+	guint version;
 	XArchive archive = {0};
 
 	if (gtk_tree_selection_get_selected (selection,&model,&iter))
 	{
-		gtk_tree_model_get(model,&iter,3,&type,-1);
+		gtk_tree_model_get(model, &iter, 3, &type, 4, &version, -1);
+
+		archive.type = type;
+		archive.version = version;
 
 		(*ask[type])(&archive);
 
@@ -205,7 +209,7 @@ static gchar *xa_multi_extract_one_archive (Multi_extract_data *dialog, gchar *f
 	gchar *new_path = NULL;
 	gchar *_filename = NULL;
 	gchar *error = NULL;
-	gint type;
+	ArchiveType xa;
 	GtkWidget *main_window;
 
 	if (dest_path == NULL)
@@ -227,8 +231,8 @@ static gchar *xa_multi_extract_one_archive (Multi_extract_data *dialog, gchar *f
 		}
 		dest_path = new_path;
 	}
-	type = xa_detect_archive_type(filename);
-	archive = xa_init_archive_structure(type);
+	xa = xa_detect_archive_type(filename);
+	archive = xa_init_archive_structure(xa);
 	dialog->archive = archive;
 	archive->do_overwrite = overwrite;
 	archive->do_full_path = full_path;
@@ -606,7 +610,7 @@ Multi_extract_data *xa_create_multi_extract_dialog()
 	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow1),GTK_POLICY_AUTOMATIC,GTK_POLICY_AUTOMATIC);
 	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolledwindow1),GTK_SHADOW_IN);
 
-	dialog_data->files_liststore = gtk_list_store_new (4,G_TYPE_STRING,G_TYPE_UINT64,G_TYPE_STRING,G_TYPE_INT);
+	dialog_data->files_liststore = gtk_list_store_new(5, G_TYPE_STRING, G_TYPE_UINT64, G_TYPE_STRING, G_TYPE_INT, G_TYPE_UINT);
 	dialog_data->files_treeview = gtk_tree_view_new_with_model(GTK_TREE_MODEL(dialog_data->files_liststore));
 	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(dialog_data->files_treeview));
 	g_signal_connect (selection,"changed",G_CALLBACK (xa_multi_extract_dialog_selection_changed),dialog_data);
@@ -715,15 +719,33 @@ void xa_multi_extract_dialog_add_file (gchar *file_path, Multi_extract_data *dia
 {
 	GtkTreeIter iter;
 	gchar *path, *path_utf8, *file, *file_utf8;
-	gint type;
+	ArchiveType xa;
+	XArchive archive = {0};
 	struct stat my_stat;
 	guint64 file_size;
 
-	type = xa_detect_archive_type(file_path);
+	xa = xa_detect_archive_type(file_path);
 
-	if (type == XARCHIVETYPE_UNKNOWN || type == XARCHIVETYPE_NOT_FOUND)
+	if (xa.type == XARCHIVETYPE_UNKNOWN || xa.type == XARCHIVETYPE_NOT_FOUND)
 		return;
 
+	if (!list[xa.type])
+	{
+		xa_show_message_dialog(GTK_WINDOW(xa_main_window), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Sorry, this archive format is not supported:"), _("The proper archiver is not installed!"));
+		return;
+	}
+
+	archive.type = xa.type;
+	archive.version = xa.version;
+
+	(*ask[xa.type])(&archive);
+
+	if (!archive.can_extract)
+	{
+		xa_show_message_dialog(GTK_WINDOW(xa_main_window), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Can't extract files from the archive:"), _("The archiver doesn't support this feature!"));
+		return;
+	}
+
 	stat (file_path,&my_stat);
 	file_size = my_stat.st_size;
 	path = xa_remove_level_from_path(file_path);
@@ -731,7 +753,7 @@ void xa_multi_extract_dialog_add_file (gchar *file_path, Multi_extract_data *dia
 	file = g_path_get_basename(file_path);
 	file_utf8 = g_filename_display_name(file);
 	gtk_list_store_append(dialog->files_liststore,&iter);
-	gtk_list_store_set(dialog->files_liststore, &iter, 0, file_utf8, 1, file_size, 2, path_utf8, 3, type, -1);
+	gtk_list_store_set(dialog->files_liststore, &iter, 0, file_utf8, 1, file_size, 2, path_utf8, 3, xa.type, 4, xa.version, -1);
 	dialog->nr++;
 	g_free(file_utf8);
 	g_free(file);
@@ -745,7 +767,7 @@ void xa_multi_extract_dialog (Multi_extract_data *dialog)
 	gchar *filename, *filename_local, *file, *path, *message, *name, *dest_path = NULL;
 	GString *output = g_string_new("");
 	gboolean overwrite = FALSE, full_path;
-	gint response,type;
+	gint response;
 	double percent = 0.0;
 
 	gtk_widget_show_all(dialog->multi_extract);
@@ -782,7 +804,7 @@ run:
 	percent=fraction;
 	do
 	{
-		gtk_tree_model_get (GTK_TREE_MODEL(dialog->files_liststore),&iter,0,&file,2,&path,3,&type,-1);
+		gtk_tree_model_get(GTK_TREE_MODEL(dialog->files_liststore), &iter, 0, &file, 2, &path, -1);
 		full_path = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(dialog->full_path));
 		filename = g_strconcat (path,"/",file,NULL);
 		xa_increase_progress_bar(progress, filename, percent);
