@@ -16,21 +16,20 @@
  *  Foundation, Inc., 59 Temple Street #330, Boston, MA 02111-1307, USA.
  */
 
-#ifndef __EXTRACT_DIALOG_H
-#define __EXTRACT_DIALOG_H
+#ifndef XARCHIVER_EXTRACT_DIALOG_H
+#define XARCHIVER_EXTRACT_DIALOG_H
 
+#include <gtk/gtk.h>
 #include "archive.h"
-#include <sys/types.h>
-#include <dirent.h>
 
-typedef struct
+typedef struct Extract_dialog_data
 {
 	GtkWidget *dialog1,*dialog_vbox1,*destination_path_entry,*overwrite_check,*extract_full,*touch,*fresh,*update,*all_files_radio,*files_radio,
 	 *selected_radio,*entry2,*password_entry;
 	gchar *string;
 } Extract_dialog_data;
 
-typedef struct
+typedef struct Multi_extract_data
 {
 	GtkWidget *multi_extract,*files_treeview,*hbox2,*extract_to,*entry1,*extract_to_archive_name,*label1,*frame2,*alignment2,*vbox3,*overwrite,*full_path,*label2,*dialog_action_area1,
 	*extract_button,*extract_image,*extract_hbox,*extract_label,*image1;
@@ -40,21 +39,11 @@ typedef struct
 	XArchive *archive;
 } Multi_extract_data;
 
-GtkWidget *label1,*label2,*label3,*label_password;
-GtkWidget *hbox1,*hbox2,*hbox3,*vbox1,*vbox2,*vbox3,*vbox5;
-GtkWidget *frame1,*frame2,*alignment1,*alignment2,*alignment3;
-GtkWidget *dialog_action_area1,*cancel_button,*okbutton1,*extract_button,*extract_image,*extract_hbox,*extract_label;
-GtkTooltips *option_tooltip;
-	
 Extract_dialog_data *xa_create_extract_dialog();
-void xa_activate_entry(GtkToggleButton *,gpointer );
-void fresh_update_toggled_cb (GtkToggleButton *, Extract_dialog_data *);
-void update_fresh_toggled_cb (GtkToggleButton *, Extract_dialog_data *);
-void xa_set_extract_dialog_options(Extract_dialog_data *,gint ,XArchive *);
-void xa_parse_extract_dialog_options (XArchive * , Extract_dialog_data *, GtkTreeSelection *);
 Multi_extract_data *xa_create_multi_extract_dialog();
-void xa_activate_remove_button (GtkTreeModel *, GtkTreePath *, GtkTreeIter *, GtkWidget *);
-void xa_add_files_liststore (gchar *, Multi_extract_data *);
-void xa_parse_multi_extract_archive(Multi_extract_data *);
-#endif
+void xa_multi_extract_dialog(Multi_extract_data *);
+void xa_multi_extract_dialog_add_file(gchar *, Multi_extract_data *);
+void xa_parse_extract_dialog_options(XArchive * , Extract_dialog_data *, GtkTreeSelection *);
+void xa_set_extract_dialog_options(Extract_dialog_data *, gint, XArchive *);
 
+#endif
