@@ -54,11 +54,11 @@ void xa_lha_ask (XArchive *archive)
 
 static void xa_lha_parse_output (gchar *line, XArchive *archive)
 {
+	XEntry *entry;
 	gpointer item[6];
+	gchar *filename;
 	unsigned int linesize,n,a;
 	gboolean dir;
-	gchar *filename;
-	XEntry *entry;
 
 	if (last_line)
 		return;
@@ -77,16 +77,15 @@ static void xa_lha_parse_output (gchar *line, XArchive *archive)
 		return;
 	}
 	linesize = strlen(line);
-	archive->files++;
 
 	/* Permission */
 	line[10] = '\0';
-	item[1] = line;
+	item[4] = line;
 	dir = (line[0] == 'd');
 
 	/* UID/GID */
 	line[22] = '\0';
-	item[2] = line + 11;
+	item[5] = line + 11;
 
 	//TODO verify the len of the size column with a big archive
 	/* Size */
@@ -100,16 +99,15 @@ static void xa_lha_parse_output (gchar *line, XArchive *archive)
 		break;
 
 	line[a+(n-a)] = '\0';
-	item[3] = line + a;
-	archive->files_size += g_ascii_strtoull(item[3],NULL,0);
+	item[1] = line + a;
 
     /* Ratio */
     line[37] = '\0';
-    item[4] = line + 31;
+    item[2] = line + 31;
 
     /* Timestamp */
     line[50] = '\0';
-    item[5] = line + 38;
+    item[3] = line + 38;
 
 	line[(linesize- 1)] = '\0';
 	filename = line + 51;
@@ -126,16 +124,24 @@ static void xa_lha_parse_output (gchar *line, XArchive *archive)
 	else
 		item[0] = NULL;
 
-	entry = xa_set_archive_entries_for_each_row (archive,filename,item);
+	entry = xa_set_archive_entries_for_each_row(archive, filename, item);
 
-	if (entry && dir)
-		entry->is_dir = TRUE;
+	if (entry)
+	{
+		if (dir)
+			entry->is_dir = TRUE;
+
+		if (!entry->is_dir)
+			archive->files++;
+
+		archive->files_size += g_ascii_strtoull(item[1], NULL, 0);
+	}
 }
 
 void xa_lha_list (XArchive *archive)
 {
-	const GType types[] = {GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_UINT64, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER};
-	const gchar *titles[] = {_("Points to"), _("Permissions"), _("UID/GID"), _("Size"), _("Ratio"), _("Timestamp")};
+	const GType types[] = {GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_UINT64, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER};
+	const gchar *titles[] = {_("Points to"), _("Original Size"), _("Occupancy"), _("Date and Time"), _("Permissions"), _("UID/GID")};
 	gchar *command;
 	guint i;
 
@@ -149,6 +155,7 @@ void xa_lha_list (XArchive *archive)
 	g_free (command);
 
 	archive->columns = 9;
+	archive->size_column = 3;
 	archive->column_types = g_malloc0(sizeof(types));
 
 	for (i = 0; i < archive->columns; i++)
