@@ -23,6 +23,7 @@
 #include "7zip.h"
 #include "add_dialog.h"
 #include "arj.h"
+#include "cpio.h"
 #include "deb.h"
 #include "extract_dialog.h"
 #include "interface.h"
@@ -34,6 +35,7 @@
 #include "string_utils.h"
 #include "support.h"
 #include "tar.h"
+#include "unar.h"
 #include "window.h"
 #include "zip.h"
 
@@ -41,6 +43,8 @@
 #include "socket.h"
 #endif
 
+#define FUNC(cond1, func1, cond2, func2, cond3, func3) (cond1 ? func1 : (cond2 ? func2 : (cond3 ? func3 : NULL)))
+
 GtkWidget *xa_main_window;
 
 XArchiver archiver[XARCHIVETYPE_TYPES];
@@ -52,7 +56,6 @@ extract_func extract[XARCHIVETYPE_TYPES];
 add_func add[XARCHIVETYPE_TYPES];
 delete_func delete[XARCHIVETYPE_TYPES];
 
-const gchar *locale;
 gchar *xdg_open;
 
 Add_dialog_data *add_window;
@@ -61,7 +64,7 @@ Multi_extract_data *multi_extract_window;
 Prefs_dialog_data *prefs_window;
 
 static gchar *opt_extract_path, *opt_compress;
-static gboolean opt_extract, opt_multi_extract, opt_add, opt_version;
+static gboolean opt_extract, opt_multi_extract, opt_add, opt_info, opt_version;
 
 static GOptionEntry entries[] =
 {
@@ -85,7 +88,11 @@ static GOptionEntry entries[] =
 		N_("Add to archive by asking which files and\n                                     quit"),
 		NULL
 	},
-	{	"version", 'V', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &opt_version,
+	{	"info", 'i', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &opt_info,
+		N_("Show found command line programs to be\n                                     used and exit"),
+		NULL
+	},
+	{	"version", 'v', G_OPTION_FLAG_IN_MAIN, G_OPTION_ARG_NONE, &opt_version,
 		N_("Show version and exit\n"), NULL },
 	{ NULL }
 };
@@ -93,36 +100,63 @@ static GOptionEntry entries[] =
 static void xa_check_available_archivers ()
 {
 	XArchiveType type;
-	gchar *path, *xz;
+	gchar *path, *cpio, *lsar, *sevenz, *unar, *xz;
+	gboolean is7z = TRUE, is7za = TRUE, is7zr = TRUE, standard;
 
 	/* (un)compressors that can handle various types */
 
+	sevenz = g_find_program_in_path("7z");
+
+	if (!sevenz)
+	{
+		is7z = FALSE;
+		sevenz = g_find_program_in_path("7za");
+
+		if (!sevenz)
+		{
+			is7za = FALSE;
+			sevenz = g_find_program_in_path("7zr");
+
+			if (!sevenz)
+				is7zr = FALSE;
+		}
+	}
+
+	cpio = g_find_program_in_path("cpio");
+	lsar = g_find_program_in_path("lsar");
+	unar = g_find_program_in_path("unar");
 	xz = g_find_program_in_path("xz");
 
 	/* 7-zip */
 
 	type = XARCHIVETYPE_7ZIP;
-	path = g_find_program_in_path("7z");
+	path = sevenz;
 
-	if (!path)
-		path = g_find_program_in_path("7za");
+	standard = (path != NULL);
 
-	if (!path)
-		path = g_find_program_in_path("7zr");
+	if (!standard)
+	{
+		if (lsar)
+			/* alternative ... */
+			path = g_strdup(lsar);
+		if (unar)
+			/* ... uncompressor */
+			archiver[type].program[1] = g_strdup(unar);
+	}
 
 	if (path)
 	{
 		archiver[type].program[0] = path;
-		archiver[type].is_compressor = TRUE;
+		archiver[type].is_compressor = standard;
 		archiver[type].type = g_slist_append(archiver[type].type, "7zip");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.7z");
 
-		ask[type] = xa_7zip_ask;
-		list[type] = xa_7zip_list;
-		test[type] = xa_7zip_test;
-		extract[type] = xa_7zip_extract;
-		add[type] = xa_7zip_add;
-		delete[type] = xa_7zip_delete;
+		ask[type] = (standard ? xa_7zip_ask : xa_unar_ask);
+		list[type] = (standard ? xa_7zip_list : xa_unar_list);
+		test[type] = (standard ? xa_7zip_test : xa_unar_test);
+		extract[type] = (standard ? xa_7zip_extract : (unar ? xa_unar_extract : NULL));
+		add[type] = (standard ? xa_7zip_add : NULL);
+		delete[type] = (standard ? xa_7zip_delete : NULL);
 	}
 
 	/* ARJ */
@@ -135,18 +169,36 @@ static void xa_check_available_archivers ()
 	else
 		path = g_find_program_in_path("unarj");
 
+	standard = (path != NULL);
+
+	if (!standard)
+	{
+		if (is7z)
+			/* alternative uncompressor */
+			path = g_strconcat(sevenz, " -tarj", NULL);
+		else
+		{
+			if (lsar)
+				/* alternative ... */
+				path = g_strdup(lsar);
+			if (unar)
+				/* ... uncompressor */
+				archiver[type].program[1] = g_strdup(unar);
+		}
+	}
+
 	if (path)
 	{
 		archiver[type].program[0] = path;
 		archiver[type].type = g_slist_append(archiver[type].type, "arj");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.arj");
 
-		ask[type] = xa_arj_ask;
-		list[type] = xa_arj_list;
-		test[type] = xa_arj_test;
-		extract[type] = xa_arj_extract;
-		add[type] = xa_arj_add;
-		delete[type] = xa_arj_delete;
+		ask[type] = FUNC(standard, xa_arj_ask, is7z, xa_7zip_ask, lsar, xa_unar_ask);
+		list[type] = FUNC(standard, xa_arj_list, is7z, xa_7zip_list, lsar, xa_unar_list);
+		test[type] = FUNC(standard, xa_arj_test, is7z, xa_7zip_test, lsar, xa_unar_test);
+		extract[type] = FUNC(standard, xa_arj_extract, is7z, xa_7zip_extract, unar, xa_unar_extract);
+		add[type] = FUNC(standard, xa_arj_add, is7z, NULL, lsar, NULL);
+		delete[type] = FUNC(standard, xa_arj_delete, is7z, NULL, lsar, NULL);
 	}
 
 	/* bzip2 */
@@ -159,17 +211,115 @@ static void xa_check_available_archivers ()
 	else
 		path = g_find_program_in_path("bunzip2");
 
+	standard = (path && archiver[type].is_compressor);
+
+	if (!standard)
+	{
+		if (is7za)
+		{
+			g_free(path);
+			/* alternative compressor */
+			path = g_strconcat(sevenz, " -tbzip2", NULL);
+			archiver[type].is_compressor = TRUE;
+		}
+		else if (!path)
+		{
+			if (lsar)
+				/* alternative ... */
+				path = g_strdup(lsar);
+			if (unar)
+				/* ... uncompressor */
+				archiver[type].program[1] = g_strdup(unar);
+		}
+		else
+			standard = TRUE;
+	}
+
 	if (path)
 	{
 		archiver[type].program[0] = path;
 		archiver[type].type = g_slist_append(archiver[type].type, "bzip2");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.bz2");
 
-		ask[type] = xa_gzip_et_al_ask;
-		list[type] = xa_gzip_et_al_list;
-		test[type] = xa_gzip_et_al_test;
-		extract[type] = xa_gzip_et_al_extract;
-		add[type] = xa_gzip_et_al_add;
+		ask[type] = FUNC(standard, xa_gzip_et_al_ask, is7za, xa_7zip_ask, lsar, xa_unar_ask);
+		list[type] = FUNC(standard, xa_gzip_et_al_list, is7za, xa_7zip_list, lsar, xa_unar_list);
+		test[type] = FUNC(standard, xa_gzip_et_al_test, is7za, xa_7zip_test, lsar, xa_unar_test);
+		extract[type] = FUNC(standard, xa_gzip_et_al_extract, is7za, xa_7zip_extract, unar, xa_unar_extract);
+		add[type] = FUNC(standard, xa_gzip_et_al_add, is7za, xa_7zip_add, lsar, NULL);
+	}
+
+	/* compress */
+
+	type = XARCHIVETYPE_COMPRESS;
+	path = g_find_program_in_path("compress");
+
+	standard = (path != NULL);
+
+	if (!standard)
+	{
+		if (is7za)
+			/* alternative uncompressor */
+			path = g_strconcat(sevenz, " -tZ", NULL);
+		else
+		{
+			if (lsar)
+				/* alternative ... */
+				path = g_strdup(lsar);
+			if (unar)
+				/* ... uncompressor */
+				archiver[type].program[1] = g_strdup(unar);
+		}
+	}
+
+	if (path)
+	{
+		archiver[type].program[0] = path;
+		archiver[type].is_compressor = standard;
+		archiver[type].type = g_slist_append(archiver[type].type, "compress");
+		archiver[type].glob = g_slist_append(archiver[type].glob, "*.Z");
+
+		ask[type] = FUNC(standard, xa_gzip_et_al_ask, is7za, xa_7zip_ask, lsar, xa_unar_ask);
+		list[type] = FUNC(standard, xa_gzip_et_al_list, is7za, xa_7zip_list, lsar, xa_unar_list);
+		test[type] = FUNC(standard, xa_gzip_et_al_test, is7za, xa_7zip_test, lsar, NULL);
+		extract[type] = FUNC(standard, xa_gzip_et_al_extract, is7za, xa_7zip_extract, unar, xa_unar_extract);
+		add[type] = FUNC(standard, xa_gzip_et_al_add, is7za, NULL, lsar, NULL);
+	}
+
+	/* cpio */
+
+	type = XARCHIVETYPE_CPIO;
+	path = cpio;
+
+	standard = (path != NULL);
+
+	if (!standard)
+	{
+		if (is7z)
+			/* alternative uncompressor */
+			path = g_strconcat(sevenz, " -tcpio", NULL);
+		else
+		{
+			if (lsar)
+				/* alternative ... */
+				path = g_strdup(lsar);
+			if (unar)
+				/* ... uncompressor */
+				archiver[type].program[1] = g_strdup(unar);
+		}
+	}
+
+	if (path)
+	{
+		archiver[type].program[0] = path;
+		archiver[type].is_compressor = standard;
+		archiver[type].type = g_slist_append(archiver[type].type, "cpio");
+		archiver[type].glob = g_slist_append(archiver[type].glob, "*.cpio");
+
+		ask[type] = FUNC(standard, xa_cpio_ask, is7z, xa_7zip_ask, lsar, xa_unar_ask);
+		list[type] = FUNC(standard, xa_cpio_list, is7z, xa_7zip_list, lsar, xa_unar_list);
+		test[type] = FUNC(standard, NULL, is7z, xa_7zip_test, lsar, NULL);
+		extract[type] = FUNC(standard, xa_cpio_extract, is7z, xa_7zip_extract, unar, xa_unar_extract);
+		add[type] = FUNC(standard, xa_cpio_add, is7z, NULL, lsar, NULL);
 	}
 
 	/* debian package */
@@ -177,15 +327,31 @@ static void xa_check_available_archivers ()
 	type = XARCHIVETYPE_DEB;
 	path = g_find_program_in_path("ar");
 
+	standard = (path != NULL);
+
+	if (!standard)
+	{
+		if (lsar && unar)
+		{
+			/* alternative ... */
+			path = g_strdup(lsar);
+			/* ... uncompressor */
+			archiver[type].program[1] = g_strdup(unar);
+		}
+		else if (is7z)
+			/* alternative uncompressor */
+			path = g_strconcat(sevenz, " -tar", NULL);
+	}
+
 	if (path)
 	{
 		archiver[type].program[0] = path;
 		archiver[type].type = g_slist_append(archiver[type].type, "deb");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.deb");
 
-		ask[type] = xa_deb_ask;
-		list[type] = xa_deb_list;
-		extract[type]  = xa_deb_extract;
+		ask[type] = FUNC(standard, xa_deb_ask, lsar && unar, xa_unar_ask, is7z, xa_7zip_ask);
+		list[type] = FUNC(standard, xa_deb_list, lsar && unar, xa_unar_list, is7z, xa_7zip_list);
+		extract[type]  = FUNC(standard, xa_deb_extract, lsar && unar, xa_unar_extract, is7z, xa_7zip_extract);
 	}
 
 	/* GNU zip */
@@ -198,12 +364,93 @@ static void xa_check_available_archivers ()
 	else
 		path = g_find_program_in_path("gunzip");
 
+	standard = (path && archiver[type].is_compressor);
+
+	if (!standard)
+	{
+		if (is7za)
+		{
+			g_free(path);
+			/* alternative compressor */
+			path = g_strconcat(sevenz, " -tgzip", NULL);
+			archiver[type].is_compressor = TRUE;
+		}
+		else if (!path)
+		{
+			if (lsar)
+				/* alternative ... */
+				path = g_strdup(lsar);
+			if (unar)
+				/* ... uncompressor */
+				archiver[type].program[1] = g_strdup(unar);
+		}
+		else
+			standard = TRUE;
+	}
+
 	if (path)
 	{
 		archiver[type].program[0] = path;
 		archiver[type].type = g_slist_append(archiver[type].type, "gzip");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.gz");
 
+		ask[type] = FUNC(standard, xa_gzip_et_al_ask, is7za, xa_7zip_ask, lsar, xa_unar_ask);
+		list[type] = FUNC(standard, xa_gzip_et_al_list, is7za, xa_7zip_list, lsar, xa_unar_list);
+		test[type] = FUNC(standard, xa_gzip_et_al_test, is7za, xa_7zip_test, lsar, xa_unar_test);
+		extract[type] = FUNC(standard, xa_gzip_et_al_extract, is7za, xa_7zip_extract, unar, xa_unar_extract);
+		add[type] = FUNC(standard, xa_gzip_et_al_add, is7za, xa_7zip_add, lsar, NULL);
+	}
+
+	/* LHA */
+
+	type = XARCHIVETYPE_LHA;
+	path = g_find_program_in_path("lha");
+
+	standard = (path != NULL);
+
+	if (!standard)
+	{
+		if (is7z)
+			/* alternative uncompressor */
+			path = g_strconcat(sevenz, " -tlzh", NULL);
+		else
+		{
+			if (lsar)
+				/* alternative ... */
+				path = g_strdup(lsar);
+			if (unar)
+				/* ... uncompressor */
+				archiver[type].program[1] = g_strdup(unar);
+		}
+	}
+
+	if (path)
+	{
+		archiver[type].program[0] = path;
+		archiver[type].is_compressor = (standard && xa_lha_check_program(path));
+		archiver[type].type = g_slist_append(archiver[type].type, "lha");
+		archiver[type].glob = g_slist_append(archiver[type].glob, "*.lzh");
+
+		ask[type] = FUNC(standard, xa_lha_ask, is7z, xa_7zip_ask, lsar, xa_unar_ask);
+		list[type] = FUNC(standard, xa_lha_list, is7z, xa_7zip_list, lsar, xa_unar_list);
+		test[type] = FUNC(standard, xa_lha_test, is7z, xa_7zip_test, lsar, xa_unar_test);
+		extract[type] = FUNC(standard, xa_lha_extract, is7z, xa_7zip_extract, unar, xa_unar_extract);
+		add[type] = FUNC(standard, xa_lha_add, is7z, NULL, lsar, NULL);
+		delete[type] = FUNC(standard, xa_lha_delete, is7z, NULL, lsar, NULL);
+	}
+
+	/* lz4 */
+
+	type = XARCHIVETYPE_LZ4;
+	path = g_find_program_in_path("lz4");
+
+	if (path)
+	{
+		archiver[type].program[0] = path;
+		archiver[type].is_compressor = TRUE;
+		archiver[type].type = g_slist_append(archiver[type].type, "lz4");
+		archiver[type].glob = g_slist_append(archiver[type].glob, "*.lz4");
+
 		ask[type] = xa_gzip_et_al_ask;
 		list[type] = xa_gzip_et_al_list;
 		test[type] = xa_gzip_et_al_test;
@@ -211,24 +458,23 @@ static void xa_check_available_archivers ()
 		add[type] = xa_gzip_et_al_add;
 	}
 
-	/* LHA */
+	/* lzip */
 
-	type = XARCHIVETYPE_LHA;
-	path = g_find_program_in_path("lha");
+	type = XARCHIVETYPE_LZIP;
+	path = g_find_program_in_path("lzip");
 
 	if (path)
 	{
 		archiver[type].program[0] = path;
-		archiver[type].is_compressor = xa_lha_check_program(path);
-		archiver[type].type = g_slist_append(archiver[type].type, "lha");
-		archiver[type].glob = g_slist_append(archiver[type].glob, "*.lzh");
+		archiver[type].is_compressor = TRUE;
+		archiver[type].type = g_slist_append(archiver[type].type, "lzip");
+		archiver[type].glob = g_slist_append(archiver[type].glob, "*.lz");
 
-		ask[type] = xa_lha_ask;
-		list[type] = xa_lha_list;
-		test[type] = xa_lha_test;
-		extract[type] = xa_lha_extract;
-		add[type] = xa_lha_add;
-		delete[type] = xa_lha_delete;
+		ask[type] = xa_gzip_et_al_ask;
+		list[type] = xa_gzip_et_al_list;
+		test[type] = xa_gzip_et_al_test;
+		extract[type] = xa_gzip_et_al_extract;
+		add[type] = xa_gzip_et_al_add;
 	}
 
 	/* lzma */
@@ -240,18 +486,36 @@ static void xa_check_available_archivers ()
 		/* alternative compressor */
 		path = g_strconcat(xz, " --format=lzma", NULL);
 
+	standard = (path != NULL);
+
+	if (!standard)
+	{
+		if (is7zr)
+			/* alternative uncompressor */
+			path = g_strconcat(sevenz, " -tlzma", NULL);
+		else
+		{
+			if (lsar)
+				/* alternative ... */
+				path = g_strdup(lsar);
+			if (unar)
+				/* ... uncompressor */
+				archiver[type].program[1] = g_strdup(unar);
+		}
+	}
+
 	if (path)
 	{
 		archiver[type].program[0] = path;
-		archiver[type].is_compressor = TRUE;
+		archiver[type].is_compressor = standard;
 		archiver[type].type = g_slist_append(archiver[type].type, "lzma");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.lzma");
 
-		ask[type] = xa_gzip_et_al_ask;
-		list[type] = xa_gzip_et_al_list;
-		test[type] = xa_gzip_et_al_test;
-		extract[type] = xa_gzip_et_al_extract;
-		add[type] = xa_gzip_et_al_add;
+		ask[type] = FUNC(standard, xa_gzip_et_al_ask, is7zr, xa_7zip_ask, lsar, xa_unar_ask);
+		list[type] = FUNC(standard, xa_gzip_et_al_list, is7zr, xa_7zip_list, lsar, xa_unar_list);
+		test[type] = FUNC(standard, xa_gzip_et_al_test, is7zr, xa_7zip_test, lsar, NULL);
+		extract[type] = FUNC(standard, xa_gzip_et_al_extract, is7zr, xa_7zip_extract, unar, xa_unar_extract);
+		add[type] = FUNC(standard, xa_gzip_et_al_add, is7zr, NULL, lsar, NULL);
 	}
 
 	/* lzop */
@@ -283,30 +547,71 @@ static void xa_check_available_archivers ()
 	else
 		path = g_find_program_in_path("unrar");
 
+	standard = (path != NULL);
+
+	if (!standard)
+	{
+		if (is7z)
+		{
+			/* alternative uncompressor */
+			path = g_strconcat(sevenz, " -trar", NULL);
+			archiver[type].program[1] = g_strconcat(sevenz, " -trar5", NULL);
+		}
+		else
+		{
+			if (lsar)
+				/* alternative ... */
+				path = g_strdup(lsar);
+			if (unar)
+				/* ... uncompressor */
+				archiver[type].program[1] = g_strdup(unar);
+		}
+	}
+
 	if (path)
 	{
 		archiver[type].program[0] = path;
 		archiver[type].type = g_slist_append(archiver[type].type, "rar");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.rar");
 
-		if (xa_rar_check_version(path) == 5)
+		if (!standard || (xa_rar_check_version(path) == 5))
 		{
 			archiver[type].type = g_slist_append(archiver[type].type, "rar5");
 			archiver[type].glob = g_slist_append(archiver[type].glob, " .rar");
+			archiver[type].version = g_slist_append(archiver[type].version, GUINT_TO_POINTER(5));
+			archiver[type].version = g_slist_append(archiver[type].version, g_slist_last(archiver[type].type)->data);
 		}
 
-		ask[type] = xa_rar_ask;
-		list[type] = xa_rar_list;
-		test[type] = xa_rar_test;
-		extract[type] = xa_rar_extract;
-		add[type] = xa_rar_add;
-		delete[type] = xa_rar_delete;
+		ask[type] = FUNC(standard, xa_rar_ask, is7z, xa_7zip_ask, lsar, xa_unar_ask);
+		list[type] = FUNC(standard, xa_rar_list, is7z, xa_7zip_list, lsar, xa_unar_list);
+		test[type] = FUNC(standard, xa_rar_test, is7z, xa_7zip_test, lsar, xa_unar_test);
+		extract[type] = FUNC(standard, xa_rar_extract, is7z, xa_7zip_extract, unar, xa_unar_extract);
+		add[type] = FUNC(standard, xa_rar_add, is7z, NULL, lsar, NULL);
+		delete[type] = FUNC(standard, xa_rar_delete, is7z, NULL, lsar, NULL);
 	}
 
 	/* RPM package */
 
 	type = XARCHIVETYPE_RPM;
-	path = g_find_program_in_path("cpio");
+	path = g_strdup(cpio);
+
+	standard = (path != NULL);
+
+	if (!standard)
+	{
+		if (is7z)
+			/* alternative uncompressor */
+			path = g_strconcat(sevenz, " -trpm", NULL);
+		else
+		{
+			if (lsar)
+				/* alternative ... */
+				path = g_strdup(lsar);
+			if (unar)
+				/* ... uncompressor */
+				archiver[type].program[1] = g_strdup(unar);
+		}
+	}
 
 	if (path)
 	{
@@ -314,9 +619,10 @@ static void xa_check_available_archivers ()
 		archiver[type].type = g_slist_append(archiver[type].type, "rpm");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.rpm");
 
-		ask[type] = xa_rpm_ask;
-		list[type] = xa_rpm_list;
-		extract[type] = xa_rpm_extract;
+		ask[type] = FUNC(standard, xa_rpm_ask, is7z, xa_7zip_ask, lsar, xa_unar_ask);
+		list[type] = FUNC(standard, xa_rpm_list, is7z, xa_7zip_list, lsar, xa_unar_list);
+		test[type] = FUNC(standard, NULL, is7z, xa_7zip_test, lsar, xa_unar_test);
+		extract[type] = FUNC(standard, xa_rpm_extract, is7z, xa_7zip_extract, unar, xa_unar_extract);
 	}
 
 	/* tape archive */
@@ -327,18 +633,37 @@ static void xa_check_available_archivers ()
 	if (!path)
 		path = g_find_program_in_path("tar");
 
+	standard = (path != NULL);
+
+	if (!standard)
+	{
+		if (is7za)
+			/* alternative compressor */
+			path = g_strconcat(sevenz, " -ttar", NULL);
+		else
+		{
+			if (lsar)
+				/* alternative ... */
+				path = g_strdup(lsar);
+			if (unar)
+				/* ... uncompressor */
+				archiver[type].program[1] = g_strdup(unar);
+		}
+	}
+
 	if (path)
 	{
 		archiver[type].program[0] = path;
-		archiver[type].is_compressor = TRUE;
+		archiver[type].is_compressor = (standard || is7za);
 		archiver[type].type = g_slist_append(archiver[type].type, "tar");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.tar");
 
-		ask[type] = xa_tar_ask;
-		list[type] = xa_tar_list;
-		extract[type] = xa_tar_extract;
-		add[type] = xa_tar_add;
-		delete[type] = xa_tar_delete;
+		ask[type] = FUNC(standard, xa_tar_ask, is7za, xa_7zip_ask, lsar, xa_unar_ask);
+		list[type] = FUNC(standard, xa_tar_list, is7za, xa_7zip_list, lsar, xa_unar_list);
+		test[type] = FUNC(standard, NULL, is7za, xa_7zip_test, lsar, NULL);
+		extract[type] = FUNC(standard, xa_tar_extract, is7za, xa_7zip_extract, unar, xa_unar_extract);
+		add[type] = FUNC(standard, xa_tar_add, is7za, xa_7zip_add, lsar, NULL);
+		delete[type] = FUNC(standard, xa_tar_delete, is7za, xa_7zip_delete, lsar, NULL);
 	}
 
 	/* xz */
@@ -346,18 +671,36 @@ static void xa_check_available_archivers ()
 	type = XARCHIVETYPE_XZ;
 	path = xz;
 
+	standard = (path != NULL);
+
+	if (!standard)
+	{
+		if (is7zr)
+			/* alternative compressor */
+			path = g_strconcat(sevenz, " -txz", NULL);
+		else
+		{
+			if (lsar)
+				/* alternative ... */
+				path = g_strdup(lsar);
+			if (unar)
+				/* ... uncompressor */
+				archiver[type].program[1] = g_strdup(unar);
+		}
+	}
+
 	if (path)
 	{
 		archiver[type].program[0] = path;
-		archiver[type].is_compressor = TRUE;
+		archiver[type].is_compressor = (standard || is7zr);
 		archiver[type].type = g_slist_append(archiver[type].type, "xz");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.xz");
 
-		ask[type] = xa_gzip_et_al_ask;
-		list[type] = xa_gzip_et_al_list;
-		test[type] = xa_gzip_et_al_test;
-		extract[type] = xa_gzip_et_al_extract;
-		add[type] = xa_gzip_et_al_add;
+		ask[type] = FUNC(standard, xa_gzip_et_al_ask, is7zr, xa_7zip_ask, lsar, xa_unar_ask);
+		list[type] = FUNC(standard, xa_gzip_et_al_list, is7zr, xa_7zip_list, lsar, xa_unar_list);
+		test[type] = FUNC(standard, xa_gzip_et_al_test, is7zr, xa_7zip_test, lsar, xa_unar_test);
+		extract[type] = FUNC(standard, xa_gzip_et_al_extract, is7zr, xa_7zip_extract, unar, xa_unar_extract);
+		add[type] = FUNC(standard, xa_gzip_et_al_add, is7zr, xa_7zip_add, lsar, NULL);
 	}
 
 	/* zip */
@@ -374,73 +717,123 @@ static void xa_check_available_archivers ()
 			archiver[type].program[1] = zip;
 			archiver[type].is_compressor = TRUE;
 		}
+	}
 
+	standard = (path && archiver[type].is_compressor);
+
+	if (!standard)
+	{
+		if (is7za)
+		{
+			g_free(path);
+			/* alternative compressor */
+			path = g_strconcat(sevenz, " -tzip", NULL);
+			archiver[type].is_compressor = TRUE;
+		}
+		else if (!path)
+		{
+			if (lsar)
+				/* alternative ... */
+				path = g_strdup(lsar);
+			if (unar)
+				/* ... uncompressor */
+				archiver[type].program[1] = g_strdup(unar);
+		}
+		else
+			standard = TRUE;
+	}
+
+	if (path)
+	{
 		archiver[type].program[0] = path;
 		archiver[type].type = g_slist_append(archiver[type].type, "zip");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.zip");
 		/* java archive */
 		archiver[type].type = g_slist_append(archiver[type].type, "jar");
 		archiver[type].glob = g_slist_append(archiver[type].glob, "*.jar");
+		archiver[type].version = g_slist_append(archiver[type].version, GUINT_TO_POINTER('j'));
+		archiver[type].version = g_slist_append(archiver[type].version, g_slist_last(archiver[type].type)->data);
 
-		ask[type] = xa_zip_ask;
-		list[type] = xa_zip_list;
-		test[type] = xa_zip_test;
-		extract[type] = xa_zip_extract;
-		add[type] = xa_zip_add;
-		delete[type] = xa_zip_delete;
+		ask[type] = FUNC(standard, xa_zip_ask, is7za, xa_7zip_ask, lsar, xa_unar_ask);
+		list[type] = FUNC(standard, xa_zip_list, is7za, xa_7zip_list, lsar, xa_unar_list);
+		test[type] = FUNC(standard, xa_zip_test, is7za, xa_7zip_test, lsar, xa_unar_test);
+		extract[type] = FUNC(standard, xa_zip_extract, is7za, xa_7zip_extract, unar, xa_unar_extract);
+		add[type] = FUNC(standard, xa_zip_add, is7za, xa_7zip_add, lsar, NULL);
+		delete[type] = FUNC(standard, xa_zip_delete, is7za, xa_7zip_delete, lsar, NULL);
 	}
 
 	/* compressed tar */
 
-	if (archiver[XARCHIVETYPE_TAR].type)
+	if (ask[XARCHIVETYPE_TAR] == xa_tar_ask || ask[XARCHIVETYPE_TAR] == xa_unar_ask)
 	{
 		struct
 		{
 			XArchiveType compressor;
-			XArchiveType tar_type;
 			gchar *type[2];
 			gchar *glob[2];
 		} compressed_tar_infos[] =
 		{
-			{XARCHIVETYPE_BZIP2, XARCHIVETYPE_TAR_BZ2, {"tar.bzip2", NULL}, {"*.tar.bz2", "*.tbz2"}},
-			{XARCHIVETYPE_GZIP, XARCHIVETYPE_TAR_GZ, {"tar.gzip", NULL}, {"*.tar.gz", "*.tgz"}},
-			{XARCHIVETYPE_LZMA, XARCHIVETYPE_TAR_LZMA, {"tar.lzma", NULL}, {"*.tar.lzma", "*.tlz"}},
-			{XARCHIVETYPE_LZOP, XARCHIVETYPE_TAR_LZOP, {"tar.lzop", NULL}, {"*.tar.lzo", "*.tzo"}},
-			{XARCHIVETYPE_XZ, XARCHIVETYPE_TAR_XZ, {"tar.xz", NULL}, {"*.tar.xz", "*.txz"}},
-			{XARCHIVETYPE_UNKNOWN, XARCHIVETYPE_UNKNOWN, {NULL, NULL}, {NULL, NULL}}
+			{XARCHIVETYPE_BZIP2, {"tar.bzip2", NULL}, {"*.tar.bz2", "*.tbz2"}},
+			{XARCHIVETYPE_COMPRESS, {"tar.Z", NULL}, {"*.tar.Z", ""}},
+			{XARCHIVETYPE_GZIP, {"tar.gzip", NULL}, {"*.tar.gz", "*.tgz"}},
+			{XARCHIVETYPE_LZ4, {"tar.lz4", NULL}, {"*.tar.lz4", ""}},
+			{XARCHIVETYPE_LZIP, {"tar.lzip", NULL}, {"*.tar.lz", ""}},
+			{XARCHIVETYPE_LZMA, {"tar.lzma", NULL}, {"*.tar.lzma", "*.tlz"}},
+			{XARCHIVETYPE_LZOP, {"tar.lzop", NULL}, {"*.tar.lzo", "*.tzo"}},
+			{XARCHIVETYPE_XZ, {"tar.xz", NULL}, {"*.tar.xz", "*.txz"}},
+			{XARCHIVETYPE_UNKNOWN, {NULL, NULL}, {NULL, NULL}}
 		}, *i;
 
 		for (i = compressed_tar_infos; i->compressor != XARCHIVETYPE_UNKNOWN; i++)
 		{
 			if (archiver[i->compressor].type)
 			{
-				type = i->tar_type;
-				archiver[type].is_compressor = archiver[i->compressor].is_compressor;
-				archiver[type].type = g_slist_append(archiver[type].type, i->type[0]);
-				archiver[type].glob = g_slist_append(archiver[type].glob, i->glob[0]);
-				archiver[type].type = g_slist_append(archiver[type].type, i->type[1]);
-				archiver[type].glob = g_slist_append(archiver[type].glob, i->glob[1]);
+				type = i->compressor;
 
-				ask[type] = xa_tar_ask;
-				list[type] = xa_tar_list;
-				extract[type] = xa_tar_extract;
-				add[type] = xa_tar_add;
-				delete[type] = xa_tar_delete;
+				if (!xa_get_compressed_tar_type(&type))
+					continue;
+
+				if ((ask[XARCHIVETYPE_TAR] == xa_tar_ask && ask[i->compressor] == xa_gzip_et_al_ask) ||
+				    (ask[XARCHIVETYPE_TAR] == xa_unar_ask && ask[i->compressor] == xa_unar_ask))
+				{
+					archiver[type].is_compressor = archiver[i->compressor].is_compressor;
+					archiver[type].type = g_slist_append(archiver[type].type, i->type[0]);
+					archiver[type].glob = g_slist_append(archiver[type].glob, i->glob[0]);
+					archiver[type].type = g_slist_append(archiver[type].type, i->type[1]);
+					archiver[type].glob = g_slist_append(archiver[type].glob, i->glob[1]);
+
+					ask[type] = ask[XARCHIVETYPE_TAR];
+					list[type] = list[XARCHIVETYPE_TAR];
+					extract[type] = extract[XARCHIVETYPE_TAR];
+					add[type] = add[XARCHIVETYPE_TAR];
+					delete[type] = delete[XARCHIVETYPE_TAR];
+				}
 			}
 		}
 	}
+
+	g_free(lsar);
+	g_free(unar);
 }
 
 static XArchive *xa_init_structure_from_cmd_line (char *filename)
 {
 	XArchive *archive;
-	XArchiveType type;
+	ArchiveType xa;
 
-	type = xa_detect_archive_type (filename);
-	if (type == XARCHIVETYPE_UNKNOWN || type == XARCHIVETYPE_NOT_FOUND)
+	xa = xa_detect_archive_type(filename);
+
+	if (xa.type == XARCHIVETYPE_UNKNOWN || xa.type == XARCHIVETYPE_NOT_FOUND)
 		return NULL;
 
-	archive = xa_init_archive_structure (type);
+	if (!list[xa.type])
+	{
+		xa_show_message_dialog(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Sorry, this archive format is not supported:"), _("The proper archiver is not installed!"));
+		return NULL;
+	}
+
+	archive = xa_init_archive_structure(xa);
+
 	if (archive == NULL)
 	{
 		xa_show_message_dialog (NULL,GTK_DIALOG_MODAL,GTK_MESSAGE_ERROR,GTK_BUTTONS_OK,_("Can't allocate memory for the archive structure!"),"" );
@@ -479,7 +872,7 @@ int main (int argc, char **argv)
 	}
 #endif
 	gtk_init_with_args(&argc, &argv, _("[ARCHIVE]"), entries, GETTEXT_PACKAGE, &cli_error);
-	g_get_charset (&locale);
+
 	if ( cli_error != NULL )
 	{
 		g_print (_("xarchiver: %s\nTry xarchiver --help to see a full list of available command line options.\n"),cli_error->message);
@@ -509,7 +902,7 @@ int main (int argc, char **argv)
 	multi_extract_window = xa_create_multi_extract_dialog();
 	xa_prefs_load_options(prefs_window);
 
-	if (opt_extract || opt_extract_path || opt_multi_extract || opt_add || opt_compress)
+	if (opt_extract || opt_extract_path || opt_multi_extract || opt_add || opt_compress || opt_info)
 	{
 		archive = xa_init_structure_from_cmd_line (argv[1]);
 		g_print(PACKAGE_NAME " 0.5.4 \xC2\xA9  " COPYRIGHT_YEAR " " COPYRIGHT_HOLDER "\n");
@@ -520,7 +913,15 @@ int main (int argc, char **argv)
 		{
 			if (!archive)
 			{
-				xa_show_message_dialog (NULL,GTK_DIALOG_MODAL,GTK_MESSAGE_ERROR,GTK_BUTTONS_OK,_("Can't extract files from the archive:"),_("You missed the archive name!\n"));
+				if (!argv[1])
+					xa_show_message_dialog(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Can't extract files from the archive:"), _("You missed the archive name!\n"));
+
+				return -1;
+			}
+
+			if (!archive->can_extract)
+			{
+				xa_show_message_dialog(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Can't extract files from the archive:"), argv[1]);
 				return -1;
 			}
 
@@ -547,7 +948,15 @@ int main (int argc, char **argv)
 		{
 			if (!archive)
 			{
-				xa_show_message_dialog (NULL,GTK_DIALOG_MODAL,GTK_MESSAGE_ERROR,GTK_BUTTONS_OK,_("Can't extract files from the archive:"),_("You missed the archive name!\n"));
+				if (!argv[1])
+					xa_show_message_dialog(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Can't extract files from the archive:"), _("You missed the archive name!\n"));
+
+				return -1;
+			}
+
+			if (!archive->can_extract)
+			{
+				xa_show_message_dialog(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Can't extract files from the archive:"), argv[1]);
 				return -1;
 			}
 
@@ -616,7 +1025,9 @@ int main (int argc, char **argv)
 		{
 			if (!archive)
 			{
-				xa_show_message_dialog (NULL,GTK_DIALOG_MODAL,GTK_MESSAGE_ERROR,GTK_BUTTONS_OK,_("Can't add files to the archive:"),_("You missed the archive name!\n"));
+				if (!argv[1])
+					xa_show_message_dialog(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Can't add files to the archive:"), _("You missed the archive name!\n"));
+
 				return -1;
 			}
 			if (!archive->can_add)
@@ -641,6 +1052,39 @@ int main (int argc, char **argv)
 			gtk_widget_destroy(add_window->dialog1);
 			g_free(add_window);
 		}
+		/* Switch -i */
+		else if (opt_info)
+		{
+			GSList *list;
+
+			for (x = XARCHIVETYPE_FIRST; x < XARCHIVETYPE_TYPES; x++)
+			{
+				list = archiver[x].type;
+
+				while (list)
+				{
+					if (list->data)
+					{
+						if (strncmp(list->data, "tar.", 4) == 0)
+							break;
+
+						g_print("%s", (char *) list->data);
+					}
+
+					list = list->next;
+
+					if (list && list->data)
+						g_print(", ");
+					else
+					{
+						g_print(":\n  %s\n", archiver[x].program[0]);
+
+						if (archiver[x].program[1])
+							g_print("  %s\n", archiver[x].program[1]);
+					}
+				}
+			}
+		}
 done:
 		if (progress)
 		{
@@ -713,6 +1157,7 @@ done:
 		g_free(archiver[x].program[1]);
 		g_slist_free(archiver[x].type);
 		g_slist_free(archiver[x].glob);
+		g_slist_free(archiver[x].version);
 	}
 
 	g_free(xdg_open);
