@@ -35,7 +35,7 @@ static gint xa_icon_name_compare_func (pixbuf_cache *a, pixbuf_cache *b)
 	return strcmp(a->icon_name, b->icon_name);
 }
 
-const char *xa_get_stock_mime_icon(char *filename)
+const char *xa_get_stock_mime_icon(const char *filename)
 {
 	const char *mime;
 	static char icon_name[80];
@@ -73,20 +73,28 @@ const char *xa_get_stock_mime_icon(char *filename)
 		return "x-office-presentation";
 	else if (strcmp(mime,"application/gzip") == 0 ||
 	         strcmp(mime,"application/vnd.debian.binary-package") == 0 ||
+	         strcmp(mime,"application/vnd.rar") == 0 || /* legacy */ strcmp(mime,"application/x-rar") == 0 ||
 	         strcmp(mime,"application/x-7z-compressed") == 0 ||
 	         strcmp(mime,"application/x-arj") == 0 ||
 	         strcmp(mime,"application/x-bzip") == 0 ||
 	         strcmp(mime,"application/x-bzip-compressed-tar") == 0 ||
+	         strcmp(mime,"application/x-compress") == 0 ||
 	         strcmp(mime,"application/x-compressed-tar") == 0 ||
+	         strcmp(mime,"application/x-cpio") == 0 ||
+	         strcmp(mime,"application/x-cpio-compressed") == 0 ||
 	         strcmp(mime,"application/x-java-archive") == 0 ||
 	         strcmp(mime,"application/x-lha") == 0 ||
+	         strcmp(mime,"application/x-lz4") == 0 ||
+	         strcmp(mime,"application/x-lz4-compressed-tar") == 0 ||
+	         strcmp(mime,"application/x-lzip") == 0 ||
+	         strcmp(mime,"application/x-lzip-compressed-tar") == 0 ||
 	         strcmp(mime,"application/x-lzma") == 0 ||
 	         strcmp(mime,"application/x-lzma-compressed-tar") == 0 ||
 	         strcmp(mime,"application/x-lzop") == 0 ||
-	         strcmp(mime,"application/x-rar") == 0 ||
 	         strcmp(mime,"application/x-rpm") == 0 ||
 	         strcmp(mime,"application/x-source-rpm") == 0 ||
 	         strcmp(mime,"application/x-tar") == 0 ||
+	         strcmp(mime,"application/x-tarz") == 0 ||
 	         strcmp(mime,"application/x-tzo") == 0 ||
 	         strcmp(mime,"application/x-xz") == 0 ||
 	         strcmp(mime,"application/x-xz-compressed-tar") == 0 ||
