@@ -25,14 +25,56 @@
 #include "support.h"
 #include "window.h"
 
+typedef struct
+{
+	gchar *name;
+	gchar *exec;
+	GdkPixbuf *icon;
+} App;
+
 typedef struct
 {
 	GtkWidget *dialog1;
 	GtkWidget *custom_command_entry;
-	GtkListStore *apps_liststore;
 	gchar *files;
+	GSList *apps;
 } Open_with_data;
 
+static gint xa_app_compare (const App *a, const App *b)
+{
+	if (strcmp(a->exec, b->exec) == 0)
+	{
+		if (a->name == b->name)
+			return 0;
+
+		if (a->name && b->name)
+			return strcmp(a->name, b->name);
+	}
+
+	return 1;
+}
+
+static void xa_app_free (App *app, gpointer user_data)
+{
+	g_free(app->name);
+	g_free(app->exec);
+	g_object_unref(app->icon);
+	g_free(app);
+}
+
+static void xa_open_with_dialog_expander_expanded (GObject *object, GParamSpec *param_spec, Open_with_data *data)
+{
+	gchar *cmd;
+
+	if (gtk_expander_get_expanded(GTK_EXPANDER(object)))
+	{
+		cmd = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(prefs_window->combo_prefered_custom_cmd));
+
+		if (cmd && *cmd)
+			gtk_entry_set_text(GTK_ENTRY(data->custom_command_entry), cmd);
+	}
+}
+
 static void xa_open_with_dialog_selection_changed (GtkTreeSelection *selection, Open_with_data *data)
 {
 	gchar *exec;
@@ -47,20 +89,44 @@ static void xa_open_with_dialog_selection_changed (GtkTreeSelection *selection,
 	}
 }
 
-static void xa_open_with_dialog_execute_command (GtkWidget *widget, Open_with_data *data)
+static void xa_open_with_dialog_execute_command (GtkButton *button, Open_with_data *data)
 {
 	const char *application;
 
 	application = gtk_entry_get_text(GTK_ENTRY(data->custom_command_entry));
-	xa_launch_external_program((gchar*)application,data->files);
+	xa_launch_external_program(application, data->files);
 	gtk_widget_destroy(data->dialog1);
 }
 
-static void xa_open_with_dialog_custom_entry_activated (GtkEditable *entry, Open_with_data *data)
+static void xa_open_with_dialog_row_selected (GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, Open_with_data *data)
 {
 	xa_open_with_dialog_execute_command(NULL, data);
 }
 
+static void xa_open_with_dialog_custom_entry_activated (GtkEntry *entry, Open_with_data *data)
+{
+	xa_open_with_dialog_execute_command(NULL, data);
+}
+
+static gboolean xa_open_with_dialog_mouse_button_event (GtkWidget *widget, GdkEventButton *event, Open_with_data *data)
+{
+	if (event->type == GDK_BUTTON_RELEASE && event->button == 2)
+	{
+		xa_open_with_dialog_execute_command(NULL, data);
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+static void xa_destroy_open_with_dialog (GtkObject *object, Open_with_data *data)
+{
+	g_free(data->files);
+	g_slist_foreach(data->apps, (GFunc) xa_app_free, NULL);
+	g_slist_free(data->apps);
+	g_free(data);
+}
+
 static void xa_open_with_dialog_browse_custom_command (GtkButton *button, Open_with_data *data)
 {
 	GtkWidget *file_selector;
@@ -89,10 +155,10 @@ static void xa_open_with_dialog_browse_custom_command (GtkButton *button, Open_w
 	gtk_widget_destroy(file_selector);
 }
 
-static void xa_parse_desktop_files (GSList **app_name_list, GSList **app_exe_list, GSList **app_icon_list, gchar *path, gchar *name)
+static void xa_parse_desktop_file (const gchar *path, const gchar *name, Open_with_data *data)
 {
 	gchar *filename, *line, *key;
-	gchar *app_name = NULL, *app_exe = NULL, *app_icon = NULL;
+	gchar *app_name = NULL, *app_exec = NULL, *app_icon = NULL;
 	GIOStatus status;
 	GIOChannel *file;
 	gboolean has_mimetype = FALSE;
@@ -138,11 +204,11 @@ static void xa_parse_desktop_files (GSList **app_name_list, GSList **app_exe_lis
 			}
 			if (g_str_has_prefix(line,"Exec="))
 			{
-				app_exe = strstr(line," %");
-				if (app_exe)
-					app_exe = g_strndup(line + 5,app_exe - (line+5));
+				app_exec = strstr(line, " %");
+				if (app_exec)
+					app_exec = g_strndup(line + 5, app_exec - (line + 5));
 				else
-					app_exe = g_strndup(line + 5,(strlen(line)-6));
+					app_exec = g_strndup(line + 5, strlen(line) - 6);
 				continue;
 			}
 			if (g_str_has_prefix(line,"Icon="))
@@ -160,47 +226,65 @@ static void xa_parse_desktop_files (GSList **app_name_list, GSList **app_exe_lis
 		}
 	}
 	while (status != G_IO_STATUS_EOF);
-	if (has_mimetype)
+
+	g_io_channel_shutdown(file, FALSE, NULL);
+
+	if (app_exec && has_mimetype)
 	{
-		*app_name_list	= g_slist_prepend(*app_name_list,app_name);
-		*app_exe_list	= g_slist_prepend(*app_exe_list ,app_exe);
-		if (app_icon == NULL)
-			app_icon = "";
-		if (gtk_combo_box_get_active(GTK_COMBO_BOX(prefs_window->combo_icon_size)) == 0)
-			size = 40;
-		else
-			size = 24;
-		*app_icon_list = g_slist_prepend(*app_icon_list, gtk_icon_theme_load_icon(icon_theme, app_icon, size, GTK_ICON_LOOKUP_FORCE_SIZE, NULL));
-		g_io_channel_shutdown (file, TRUE, NULL);
-		return;
+		App app;
+		GSList *duplicate;
+
+		app.name = app_name;
+		app.exec = app_exec;
+		app.icon = NULL;
+
+		duplicate = g_slist_find_custom(data->apps, &app, (GCompareFunc) xa_app_compare);
+
+		if (!duplicate)
+		{
+			App *app;
+			GdkPixbuf *pixbuf = NULL;
+
+			app = g_new0(App, 1);
+
+			app->name = app_name;
+			app->exec = app_exec;
+
+			if (gtk_combo_box_get_active(GTK_COMBO_BOX(prefs_window->combo_icon_size)) == 0)
+				size = 40;
+			else
+				size = 24;
+
+			if (app_icon)
+			{
+				pixbuf = gtk_icon_theme_load_icon(icon_theme, app_icon, size, GTK_ICON_LOOKUP_FORCE_SIZE, NULL);
+				g_free(app_icon);
+			}
+
+			if (!pixbuf)
+			{
+				pixbuf = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, size, size);
+				gdk_pixbuf_fill(pixbuf, 0x00000000);
+			}
+
+			app->icon = pixbuf;
+
+			data->apps = g_slist_prepend(data->apps, app);
+
+			return;
+		}
 	}
-	if (app_name != NULL)
-	{
-		g_free(app_name);
-		app_name = NULL;
-	}
-	if (app_exe != NULL)
-	{
-		g_free(app_exe);
-		app_exe = NULL;
-	}
-	if (app_icon != NULL)
-	{
-		g_free(app_icon);
-		app_icon = NULL;
-	}
-	g_io_channel_shutdown (file, TRUE, NULL);
+
+	g_free(app_name);
+	g_free(app_exec);
+	g_free(app_icon);
 }
 
-static void xa_read_desktop_directories (GtkListStore *liststore, const gchar *dirname)
+static void xa_read_desktop_directory (const gchar *dirname, GtkListStore *liststore, Open_with_data *data)
 {
 	DIR *dir;
 	gchar *filename = NULL;
 	struct dirent *dirlist;
-	GSList *app_icon = NULL;
-	GSList *app_name  = NULL;
-	GSList *app_exe = NULL;
-	GtkTreeIter iter;
 
 	filename = g_build_filename(dirname,"applications",NULL);
 	dir = opendir(filename);
@@ -210,52 +294,32 @@ static void xa_read_desktop_directories (GtkListStore *liststore, const gchar *d
 		g_free(filename);
 		return;
 	}
+
 	while ((dirlist = readdir(dir)))
 	{
 		if (g_str_has_suffix(dirlist->d_name,".desktop"))
-			xa_parse_desktop_files(&app_name,&app_exe,&app_icon,filename,dirlist->d_name);
+			xa_parse_desktop_file(filename, dirlist->d_name, data);
 	}
-	while (app_name)
-	{
-		gtk_list_store_append(liststore,&iter);
-		gtk_list_store_set(liststore,&iter,0,app_icon->data,1,app_name->data,2,app_exe->data,-1);
-		if (app_icon->data != NULL)
-			g_object_unref(app_icon->data);
-		app_name = app_name->next;
-		app_icon = app_icon->next;
-		app_exe  = app_exe->next;
-	}
-	g_free(filename);
+
 	closedir(dir);
-
-	g_slist_foreach(app_name,(GFunc)g_free,NULL);
-	g_slist_foreach(app_exe,(GFunc)g_free,NULL);
-	g_slist_free(app_name);
-	g_slist_free(app_exe);
+	g_free(filename);
 }
 
-static void xa_open_with_dialog_row_selected (GtkTreeView *tree_view, GtkTreePath *path, GtkTreeViewColumn *column, Open_with_data *data)
-{
-	xa_open_with_dialog_execute_command(NULL, data);
-}
-
-static void xa_destroy_open_with_dialog (GtkWidget *widget, Open_with_data *data)
-{
-	g_free(data);
-}
-
-void xa_create_open_with_dialog(gchar *filename,gchar *filenames,int nr)
+void xa_create_open_with_dialog (const gchar *filename, gchar *filenames, gint nr)
 {
 	Open_with_data *data = NULL;
+	GtkListStore *apps_liststore;
 	GtkWidget	*dialog_vbox1,*vbox1,*hbox1,*mime_icon,*open_text,*scrolledwindow1,*apps_treeview,*dialog_action_area1,
 				*custom_command_expander,*hbox_expander,*browse,*cancelbutton1,*okbutton1;
 	GtkCellRenderer		*renderer;
 	GtkTreeViewColumn	*column;
 	GtkTreeIter iter;
 	GdkPixbuf *pixbuf;
+	GSList *apps;
 	gchar *text = NULL;
 	gchar *title;
 	const gchar *icon_name = NULL;
+	const gchar *desktop_dir;
 	const gchar* const *desktop_dirs;
 	gint x = 0;
 
@@ -303,8 +367,8 @@ void xa_create_open_with_dialog(gchar *filename,gchar *filenames,int nr)
 	gtk_box_pack_start (GTK_BOX (vbox1),scrolledwindow1,TRUE,TRUE,0);
 	g_object_set (G_OBJECT (scrolledwindow1),"hscrollbar-policy",GTK_POLICY_AUTOMATIC,"shadow-type",GTK_SHADOW_IN,"vscrollbar-policy",GTK_POLICY_AUTOMATIC,NULL);
 
-	data->apps_liststore = gtk_list_store_new (3,GDK_TYPE_PIXBUF,G_TYPE_STRING,G_TYPE_STRING);
-	apps_treeview = gtk_tree_view_new_with_model(GTK_TREE_MODEL(data->apps_liststore));
+	apps_liststore = gtk_list_store_new(3, GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING);
+	apps_treeview = gtk_tree_view_new_with_model(GTK_TREE_MODEL(apps_liststore));
 	gtk_container_add (GTK_CONTAINER (scrolledwindow1),apps_treeview);
 	gtk_tree_view_set_headers_visible(GTK_TREE_VIEW(apps_treeview),FALSE);
 	GtkTreeSelection *sel = gtk_tree_view_get_selection( GTK_TREE_VIEW (apps_treeview));
@@ -320,7 +384,7 @@ void xa_create_open_with_dialog(gchar *filename,gchar *filenames,int nr)
 	gtk_tree_view_column_pack_start(column,renderer, TRUE);
 	gtk_tree_view_column_set_attributes( column,renderer,"text",1,NULL);
 	gtk_tree_view_column_set_resizable (column, TRUE);
-	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(data->apps_liststore),1,GTK_SORT_ASCENDING);
+	gtk_tree_sortable_set_sort_column_id(GTK_TREE_SORTABLE(apps_liststore), 1, GTK_SORT_ASCENDING);
 	gtk_tree_view_append_column (GTK_TREE_VIEW (apps_treeview), column);
 	gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
 
@@ -330,6 +394,7 @@ void xa_create_open_with_dialog(gchar *filename,gchar *filenames,int nr)
 	gtk_tree_view_append_column (GTK_TREE_VIEW (apps_treeview), column);
 
 	custom_command_expander = gtk_expander_new_with_mnemonic(_("Use a custom command:"));
+	g_signal_connect(G_OBJECT(custom_command_expander), "notify::expanded", G_CALLBACK(xa_open_with_dialog_expander_expanded), data);
 	gtk_box_pack_start (GTK_BOX (vbox1),custom_command_expander,FALSE,FALSE,0);
 
 	hbox_expander = gtk_hbox_new(FALSE,5);
@@ -360,16 +425,37 @@ void xa_create_open_with_dialog(gchar *filename,gchar *filenames,int nr)
 	gtk_widget_set_can_default(okbutton1, TRUE);
 	gtk_widget_show_all(data->dialog1);
 
-	/* Let's parse the desktop files in all the system data dirs */
+	/* let's parse the desktop files in the user data dir */
+	desktop_dir = g_get_user_data_dir();
+
+	if (desktop_dir)
+		xa_read_desktop_directory(desktop_dir, apps_liststore, data);
+
+	/* let's parse the desktop files in all the system data dirs */
 	desktop_dirs = g_get_system_data_dirs();
+
 	while (desktop_dirs[x])
 	{
-		xa_read_desktop_directories(data->apps_liststore,desktop_dirs[x]);
+		xa_read_desktop_directory(desktop_dirs[x], apps_liststore, data);
 		x++;
 	}
-	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(data->apps_liststore),&iter);
+
+	apps = data->apps;
+
+	while (apps)
+	{
+		App *app = apps->data;
+
+		gtk_list_store_append(apps_liststore, &iter);
+		gtk_list_store_set(apps_liststore, &iter, 0, app->icon, 1, app->name, 2, app->exec, -1);
+
+		apps = apps->next;
+	}
+
+	gtk_tree_model_get_iter_first(GTK_TREE_MODEL(apps_liststore), &iter);
 	gtk_tree_selection_select_iter(gtk_tree_view_get_selection (GTK_TREE_VIEW (apps_treeview)),&iter);
 
 	g_signal_connect (G_OBJECT (apps_treeview),	"row-activated",G_CALLBACK(xa_open_with_dialog_row_selected),data);
+	g_signal_connect(G_OBJECT(apps_treeview), "button-release-event", G_CALLBACK(xa_open_with_dialog_mouse_button_event), data);
 	g_signal_connect (G_OBJECT (data->dialog1),	"destroy",		G_CALLBACK(xa_destroy_open_with_dialog),data);
 }
