@@ -27,35 +27,6 @@
 gchar *config_file;
 GtkIconTheme *icon_theme;
 
-static void xa_prefs_iconview_changed (GtkIconView *iconview, gpointer data)
-{
-	Prefs_dialog_data *prefs = data;
-	GList *list;
-	GtkTreePath *path;
-	GtkTreeIter iter;
-	guint column = 0;
-
-	list = gtk_icon_view_get_selected_items (iconview);
-	if (list == NULL)
-		return;
-
-	list = g_list_first (list);
-	path = (GtkTreePath*)list->data;
-
-	gtk_tree_model_get_iter (GTK_TREE_MODEL(prefs->prefs_liststore),&iter,path);
-	gtk_tree_model_get (GTK_TREE_MODEL(prefs->prefs_liststore),&iter,2,&column,-1);
-
-	gtk_tree_path_free(path);
-	g_list_free (list);
-
-	if (column == 0)
-		gtk_notebook_set_current_page (GTK_NOTEBOOK(prefs->prefs_notebook),0);
-	else if (column == 1)
-		gtk_notebook_set_current_page (GTK_NOTEBOOK(prefs->prefs_notebook),1);
-	else if (column == 2)
-		gtk_notebook_set_current_page (GTK_NOTEBOOK(prefs->prefs_notebook),2);
-}
-
 static gchar *xa_prefs_choose_program (gboolean flag)
 {
 	gchar *filename = NULL;
@@ -112,7 +83,9 @@ static void xa_prefs_dialog_set_default_options (Prefs_dialog_data *prefs_data)
 		gtk_combo_box_set_active (GTK_COMBO_BOX(prefs_data->combo_prefered_web_browser),0);
 		gtk_combo_box_set_active (GTK_COMBO_BOX(prefs_data->combo_prefered_editor),0);
 		gtk_combo_box_set_active (GTK_COMBO_BOX(prefs_data->combo_prefered_viewer),0);
+		gtk_combo_box_set_active(GTK_COMBO_BOX(prefs_data->combo_prefered_archiver), 0);
 	}
+	gtk_combo_box_set_active(GTK_COMBO_BOX(prefs_data->combo_prefered_custom_cmd), 0);
 	gtk_combo_box_set_active (GTK_COMBO_BOX(prefs_data->combo_prefered_temp_dir),0);
 	gtk_combo_box_set_active (GTK_COMBO_BOX(prefs_data->combo_prefered_extract_dir),0);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prefs_data->check_save_geometry),FALSE);
@@ -123,13 +96,38 @@ static void xa_prefs_dialog_set_default_options (Prefs_dialog_data *prefs_data)
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (add_window->recurse),TRUE);
 }
 
+void xa_prefs_iconview_changed (GtkIconView *iconview, gpointer data)
+{
+	Prefs_dialog_data *prefs = data;
+	GList *list;
+	GtkTreePath *path;
+	GtkTreeIter iter;
+	guint column = 0;
+
+	list = gtk_icon_view_get_selected_items (iconview);
+	if (list == NULL)
+		return;
+
+	list = g_list_first (list);
+	path = (GtkTreePath*)list->data;
+
+	gtk_tree_model_get_iter (GTK_TREE_MODEL(prefs->prefs_liststore),&iter,path);
+	gtk_tree_model_get (GTK_TREE_MODEL(prefs->prefs_liststore),&iter,2,&column,-1);
+
+	gtk_tree_path_free(path);
+	g_list_free (list);
+
+	gtk_notebook_set_current_page(GTK_NOTEBOOK(prefs->prefs_notebook), column);
+}
+
 Prefs_dialog_data *xa_create_prefs_dialog()
 {
 	GTK_COMPAT_TOOLTIPS;
-	GtkWidget *vbox1, *vbox3,*vbox4, *hbox1, *scrolledwindow1, *prefs_iconview;
+	GtkWidget *vbox1, *vbox3, *vbox4, *hbox1, *scrolledwindow1;
 	GtkWidget *label1, *label2, *label3, *label4, *label5,*label6, *label7, *label8, *label9, *label10, *table1, *table2;
 	GtkTreeIter iter;
 	GdkPixbuf *icon_pixbuf;
+	GtkTreePath *top;
 	Prefs_dialog_data *prefs_data;
 
 	prefs_data = g_new0 (Prefs_dialog_data,1);
@@ -168,19 +166,23 @@ Prefs_dialog_data *xa_create_prefs_dialog()
 	gtk_list_store_set (prefs_data->prefs_liststore, &iter, 0, icon_pixbuf, 1, _("Advanced"),2,2,-1);
 	g_object_unref (icon_pixbuf);
 
-	prefs_iconview = gtk_icon_view_new_with_model(GTK_TREE_MODEL(prefs_data->prefs_liststore));
+	prefs_data->iconview = gtk_icon_view_new_with_model(GTK_TREE_MODEL(prefs_data->prefs_liststore));
 	g_object_unref (prefs_data->prefs_liststore);
 
-	gtk_icon_view_set_item_orientation(GTK_ICON_VIEW (prefs_iconview), GTK_ORIENTATION_VERTICAL);
-	gtk_icon_view_set_columns (GTK_ICON_VIEW (prefs_iconview),1);
-	gtk_icon_view_set_pixbuf_column (GTK_ICON_VIEW (prefs_iconview), 0);
-	gtk_icon_view_set_text_column(GTK_ICON_VIEW (prefs_iconview),1);
-	gtk_container_add (GTK_CONTAINER (scrolledwindow1), prefs_iconview);
+	gtk_icon_view_set_item_orientation(GTK_ICON_VIEW(prefs_data->iconview), GTK_ORIENTATION_VERTICAL);
+	gtk_icon_view_set_columns(GTK_ICON_VIEW(prefs_data->iconview), 1);
+	gtk_icon_view_set_pixbuf_column(GTK_ICON_VIEW(prefs_data->iconview), 0);
+	gtk_icon_view_set_text_column(GTK_ICON_VIEW(prefs_data->iconview), 1);
+	gtk_container_add(GTK_CONTAINER(scrolledwindow1), prefs_data->iconview);
+
+	top = gtk_tree_path_new_from_string("0");
+	gtk_icon_view_select_path(GTK_ICON_VIEW(prefs_data->iconview), top);
+	gtk_tree_path_free(top);
 
 	prefs_data->prefs_notebook = gtk_notebook_new ();
 	g_object_set (G_OBJECT (prefs_data->prefs_notebook),"show-border", FALSE,"show-tabs", FALSE,"enable-popup",FALSE,NULL);
 	gtk_box_pack_start (GTK_BOX (hbox1), prefs_data->prefs_notebook,TRUE,TRUE,0);
-	g_signal_connect (G_OBJECT (prefs_iconview),"selection-changed",G_CALLBACK (xa_prefs_iconview_changed),prefs_data);
+	g_signal_connect(G_OBJECT(prefs_data->iconview), "selection-changed", G_CALLBACK(xa_prefs_iconview_changed), prefs_data);
 
 	/* Archive page*/
 	vbox4 = gtk_vbox_new (FALSE, 2);
@@ -265,7 +267,7 @@ Prefs_dialog_data *xa_create_prefs_dialog()
   	gtk_container_add (GTK_CONTAINER (prefs_data->prefs_notebook), vbox3);
 
 
-	table2 = gtk_table_new (7, 2,FALSE);
+	table2 = gtk_table_new(9, 2, FALSE);
 	gtk_box_pack_start (GTK_BOX (vbox3), table2, TRUE, TRUE, 0);
 	gtk_table_set_row_spacings (GTK_TABLE (table2), 1);
 	gtk_table_set_col_spacings (GTK_TABLE (table2), 4);
@@ -310,9 +312,36 @@ Prefs_dialog_data *xa_create_prefs_dialog()
 		gtk_table_attach (GTK_TABLE (table2), prefs_data->combo_prefered_viewer, 1, 2, 2, 3,
 					(GtkAttachOptions) (GTK_FILL),
 					(GtkAttachOptions) (GTK_SHRINK), 0, 0);
+
+		label8 = gtk_label_new(_("Open archive files with:"));
+		gtk_table_attach(GTK_TABLE(table2), label8, 0, 1, 3, 4,
+		                 (GtkAttachOptions) (GTK_FILL),
+		                 (GtkAttachOptions) (GTK_SHRINK), 0, 0);
+		gtk_misc_set_alignment(GTK_MISC(label8), 0, 0.5);
+		prefs_data->combo_prefered_archiver = gtk_combo_box_text_new();
+		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(prefs_data->combo_prefered_archiver), "");
+		gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(prefs_data->combo_prefered_archiver), _("choose..."));
+		g_signal_connect(prefs_data->combo_prefered_archiver, "changed", G_CALLBACK(xa_prefs_combo_changed), NULL);
+		gtk_table_attach(GTK_TABLE(table2), prefs_data->combo_prefered_archiver, 1, 2, 3, 4,
+		                 (GtkAttachOptions) (GTK_FILL),
+		                 (GtkAttachOptions) (GTK_SHRINK), 0, 0);
 	}
+
+	label8 = gtk_label_new(_("Default custom command:"));
+	gtk_table_attach(GTK_TABLE(table2), label8, 0, 1, 4, 5,
+	                 (GtkAttachOptions) (GTK_FILL),
+	                 (GtkAttachOptions) (GTK_SHRINK), 0, 0);
+	gtk_misc_set_alignment(GTK_MISC(label8), 0, 0.5);
+	prefs_data->combo_prefered_custom_cmd = gtk_combo_box_text_new();
+	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(prefs_data->combo_prefered_custom_cmd), "");
+	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(prefs_data->combo_prefered_custom_cmd), _("choose..."));
+	g_signal_connect(prefs_data->combo_prefered_custom_cmd, "changed", G_CALLBACK(xa_prefs_combo_changed), NULL);
+	gtk_table_attach(GTK_TABLE(table2), prefs_data->combo_prefered_custom_cmd, 1, 2, 4, 5,
+	                 (GtkAttachOptions) (GTK_FILL),
+	                 (GtkAttachOptions) (GTK_SHRINK), 0, 0);
+
 	label9 = gtk_label_new (_("Preferred temp directory:"));
-	gtk_table_attach (GTK_TABLE (table2), label9, 0, 1, 3, 4,
+	gtk_table_attach(GTK_TABLE(table2), label9, 0, 1, 5, 6,
 					(GtkAttachOptions) (GTK_FILL),
 					(GtkAttachOptions) (GTK_SHRINK), 0, 0);
 	gtk_misc_set_alignment (GTK_MISC (label9), 0, 0.5);
@@ -320,12 +349,12 @@ Prefs_dialog_data *xa_create_prefs_dialog()
 	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(prefs_data->combo_prefered_temp_dir), _("/tmp"));
 	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(prefs_data->combo_prefered_temp_dir), _("choose..."));
 	g_signal_connect (prefs_data->combo_prefered_temp_dir,"changed",G_CALLBACK (xa_prefs_combo_changed),GUINT_TO_POINTER(1));
-	gtk_table_attach (GTK_TABLE (table2), prefs_data->combo_prefered_temp_dir, 1, 2, 3, 4,
+	gtk_table_attach(GTK_TABLE(table2), prefs_data->combo_prefered_temp_dir, 1, 2, 5, 6,
 					(GtkAttachOptions) (GTK_FILL),
 					(GtkAttachOptions) (GTK_SHRINK), 0, 0);
 
 	label10 = gtk_label_new (_("Preferred extraction directory:"));
-	gtk_table_attach (GTK_TABLE (table2), label10, 0, 1, 4, 5,
+	gtk_table_attach(GTK_TABLE(table2), label10, 0, 1, 6, 7,
 					(GtkAttachOptions) (GTK_FILL),
 					(GtkAttachOptions) (GTK_SHRINK), 0, 0);
 	gtk_misc_set_alignment (GTK_MISC (label10), 0, 0.5);
@@ -333,17 +362,17 @@ Prefs_dialog_data *xa_create_prefs_dialog()
 	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(prefs_data->combo_prefered_extract_dir), _("/tmp"));
 	gtk_combo_box_text_append_text(GTK_COMBO_BOX_TEXT(prefs_data->combo_prefered_extract_dir), _("choose..."));
 	g_signal_connect (prefs_data->combo_prefered_extract_dir,"changed",G_CALLBACK (xa_prefs_combo_changed),GUINT_TO_POINTER(1));
-	gtk_table_attach (GTK_TABLE (table2), prefs_data->combo_prefered_extract_dir, 1, 2, 4, 5,
+	gtk_table_attach(GTK_TABLE(table2), prefs_data->combo_prefered_extract_dir, 1, 2, 6, 7,
 					(GtkAttachOptions) (GTK_FILL),
 					(GtkAttachOptions) (GTK_SHRINK), 0, 0);
 
 	prefs_data->check_save_geometry = gtk_check_button_new_with_mnemonic (_("Save window geometry"));
-	gtk_table_attach (GTK_TABLE (table2), prefs_data->check_save_geometry, 0, 2, 5, 6,
+	gtk_table_attach(GTK_TABLE(table2), prefs_data->check_save_geometry, 0, 2, 7, 8,
 					(GtkAttachOptions) (GTK_FILL),
 					(GtkAttachOptions) (GTK_FILL), 0, 0);
 
 	prefs_data->allow_sub_dir = gtk_check_button_new_with_mnemonic (_("Allow subdirs with drag and drop"));
-	gtk_table_attach (GTK_TABLE (table2), prefs_data->allow_sub_dir, 0, 2, 6, 7,
+	gtk_table_attach(GTK_TABLE(table2), prefs_data->allow_sub_dir, 0, 2, 8, 9,
 					(GtkAttachOptions) (GTK_FILL),
 					(GtkAttachOptions) (0), 0, 0);
 	gtk_widget_set_tooltip_text(prefs_data->allow_sub_dir, _("This option includes the subdirectories when you add files with drag and drop"));
@@ -398,7 +427,24 @@ void xa_prefs_save_options(Prefs_dialog_data *prefs_data, const char *filename)
 			g_key_file_set_string (xa_key_file,PACKAGE,"preferred_viewer",value);
 			g_free(value);
 		}
+
+		value = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(prefs_data->combo_prefered_archiver));
+
+		if (value != NULL)
+		{
+			g_key_file_set_string(xa_key_file, PACKAGE, "preferred_archiver", value);
+			g_free(value);
+		}
 	}
+
+	value = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(prefs_data->combo_prefered_custom_cmd));
+
+	if (value != NULL)
+	{
+		g_key_file_set_string(xa_key_file, PACKAGE, "preferred_custom_cmd", value);
+		g_free(value);
+	}
+
 	value = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(prefs_data->combo_prefered_temp_dir));
 	if (value != NULL)
 	{
@@ -526,7 +572,31 @@ void xa_prefs_load_options(Prefs_dialog_data *prefs_data)
 				gtk_combo_box_set_active (GTK_COMBO_BOX(prefs_data->combo_prefered_viewer),0);
 				g_free(value);
 			}
+
+			value = g_key_file_get_string(xa_key_file, PACKAGE, "preferred_archiver", NULL);
+
+			if (!value)
+				value = g_find_program_in_path("xarchiver");
+
+			if (value != NULL)
+			{
+				gtk_combo_box_text_remove(GTK_COMBO_BOX_TEXT(prefs_data->combo_prefered_archiver), 0);
+				gtk_combo_box_text_insert_text(GTK_COMBO_BOX_TEXT(prefs_data->combo_prefered_archiver), 0, value);
+				gtk_combo_box_set_active(GTK_COMBO_BOX(prefs_data->combo_prefered_archiver), 0);
+				g_free(value);
+			}
 		}
+
+		value = g_key_file_get_string(xa_key_file, PACKAGE, "preferred_custom_cmd", NULL);
+
+		if (value != NULL)
+		{
+			gtk_combo_box_text_remove(GTK_COMBO_BOX_TEXT(prefs_data->combo_prefered_custom_cmd), 0);
+			gtk_combo_box_text_insert_text(GTK_COMBO_BOX_TEXT(prefs_data->combo_prefered_custom_cmd), 0, value);
+			gtk_combo_box_set_active(GTK_COMBO_BOX(prefs_data->combo_prefered_custom_cmd), 0);
+			g_free(value);
+		}
+
 		value = g_key_file_get_string(xa_key_file,PACKAGE,"preferred_temp_dir",NULL);
 		if (value != NULL)
 		{
