@@ -21,24 +21,26 @@
 
 #include <gtk/gtk.h>
 
-extern gchar *config_file;
-extern GtkIconTheme *icon_theme;
-
 typedef struct Prefs_dialog_data
 {
 	GtkWidget *dialog1,*dialog_vbox1,*combo_prefered_format;
 	GtkWidget *confirm_deletion, *store_output,*combo_archive_view,*combo_icon_size;
-	GtkWidget *check_show_comment, *check_sort_filename_column,*show_location_bar,*show_sidebar,*show_toolbar,*combo_prefered_viewer;
+	GtkWidget *check_show_comment, *check_sort_filename_column, *show_location_bar, *show_sidebar, *show_toolbar, *combo_prefered_viewer, *combo_prefered_archiver, *combo_prefered_custom_cmd;
 	GtkWidget *combo_prefered_web_browser, *combo_prefered_editor, *combo_prefered_temp_dir, *combo_prefered_extract_dir, *allow_sub_dir,*check_save_geometry,*prefs_notebook;
 	GtkListStore *prefs_liststore;
+	GtkWidget *iconview;
 	gint geometry[5];
 	gint extract_dialog[2];
 	gint add_coords[2];
 	gboolean size_changed[2];
 } Prefs_dialog_data;
 
+extern gchar *config_file;
+extern GtkIconTheme *icon_theme;
+
 void xa_apply_prefs_option(Prefs_dialog_data *);
 Prefs_dialog_data *xa_create_prefs_dialog();
+void xa_prefs_iconview_changed(GtkIconView *, gpointer);
 void xa_prefs_load_options(Prefs_dialog_data *);
 void xa_prefs_save_options(Prefs_dialog_data *, const char *);
 
