@@ -150,14 +150,12 @@ static void xa_rar_parse_output (gchar *line, XArchive *archive)
 	else
 	{
 		linesize = strlen(line);
-		archive->files++;
 		/* Size */
 		for(n=0; n < linesize && line[n] == ' '; n++);
 		a = n;
 		for(; n < linesize && line[n] != ' '; n++);
 		line[n]='\0';
 		item[i] = line + a;
-		archive->files_size += g_ascii_strtoull(item[i],NULL,0);
 		i++;
 		n++;
 
@@ -234,14 +232,21 @@ static void xa_rar_parse_output (gchar *line, XArchive *archive)
 		line[n] = '\0';
 		item[i] = line + a;
 
-		entry = xa_set_archive_entries_for_each_row (archive,filename,item);
-		if (entry != NULL)
+		entry = xa_set_archive_entries_for_each_row(archive, filename, item);
+
+		if (entry)
 		{
 			if (dir)
 				entry->is_dir = TRUE;
 
 			entry->is_encrypted = encrypted;
+
+			if (!entry->is_dir)
+				archive->files++;
+
+			archive->files_size += g_ascii_strtoull(item[0], NULL, 0);
 		}
+
 		g_free(filename);
 		fname_line = FALSE;
 	}
@@ -291,8 +296,6 @@ static void xa_rar5_parse_output (gchar *line, XArchive *archive)
 			}
 			return;
 		}
-		if (strncmp(line, "Details: RAR 5", 14) == 0)
-			archive->version = 5;
 		if (line[0] == '-')
 		{
 			data_line = TRUE;
@@ -314,7 +317,6 @@ static void xa_rar5_parse_output (gchar *line, XArchive *archive)
 		last_line = TRUE;
 		return;
 	}
-	archive->files++;
 
 	/* Permissions */
 	for (n = encrypted ? 1 : 0; n < linesize && line[n] == ' '; n++);
@@ -333,7 +335,6 @@ static void xa_rar5_parse_output (gchar *line, XArchive *archive)
 	for(; n < linesize && line[n] != ' '; n++);
 	line[n]='\0';
 	item[i] = line + a;
-	archive->files_size += g_ascii_strtoull(item[i],NULL,0);
 	i++;
 	n++;
 
@@ -391,13 +392,20 @@ static void xa_rar5_parse_output (gchar *line, XArchive *archive)
 	*(end + 1) = '\0';
 
 	entry = xa_set_archive_entries_for_each_row (archive,filename,item);
-	if (entry != NULL)
+
+	if (entry)
 	{
 		if (dir)
 			entry->is_dir = TRUE;
 
 		entry->is_encrypted = encrypted;
+
+		if (!entry->is_dir)
+			archive->files++;
+
+		archive->files_size += g_ascii_strtoull(item[0], NULL, 0);
 	}
+
 	g_free(filename);
 }
 
@@ -474,13 +482,14 @@ void xa_rar_list (XArchive *archive)
 	if (rar_version == 5)
 	{
 		const GType types[] = {GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_UINT64, G_TYPE_UINT64, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER};
-		const gchar *titles[] = {_("Original"), _("Compressed"), _("Ratio"), _("Date"), _("Time"), _("Permissions"), _("CRC")};
+		const gchar *titles[] = {_("Original Size"), _("Compressed"), _("Occupancy"), _("Date"), _("Time"), _("Attributes"), _("Checksum")};
 
 		archive->parse_output = xa_rar5_parse_output;
 		xa_spawn_async_process (archive,command);
 		g_free ( command );
 
 		archive->columns = 10;
+		archive->size_column = 2;
 		archive->column_types = g_malloc0(sizeof(types));
 
 		for (i = 0; i < archive->columns; i++)
@@ -491,13 +500,14 @@ void xa_rar_list (XArchive *archive)
 	else
 	{
 		const GType types[] = {GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_UINT64, G_TYPE_UINT64, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER};
-		const gchar *titles[] = {_("Original"), _("Compressed"), _("Ratio"), _("Date"), _("Time"), _("Permissions"), _("CRC"), _("Method"), _("Version")};
+		const gchar *titles[] = {_("Original Size"), _("Compressed"), _("Occupancy"), _("Date"), _("Time"), _("Attributes"), _("Checksum"), _("Method"), _("Version")};
 
 		archive->parse_output = xa_rar_parse_output;
 		xa_spawn_async_process (archive,command);
 		g_free ( command );
 
 		archive->columns = 12;
+		archive->size_column = 2;
 		archive->column_types = g_malloc0(sizeof(types));
 
 		for (i = 0; i < archive->columns; i++)
