@@ -25,7 +25,7 @@
 
 gboolean isTar (FILE *file)
 {
-	unsigned char magic[7];
+	unsigned char magic[8];
 
 	fseek(file, 0, SEEK_SET);
 
@@ -38,9 +38,9 @@ gboolean isTar (FILE *file)
 
 	fseek(file, 0, SEEK_SET);
 
-	return (memcmp(magic, "\x75\x73\x74\x61\x72\x00\x30", sizeof(magic)) == 0 ||
-	        memcmp(magic, "\x75\x73\x74\x61\x72\x20\x20", sizeof(magic)) == 0 ||
-	        memcmp(magic, "\x0\x0\x0\x0\x0\x0\x0", sizeof(magic)) == 0);
+	return (memcmp(magic, "\x75\x73\x74\x61\x72\x00\x30\x30", sizeof(magic)) == 0 ||
+	        memcmp(magic, "\x75\x73\x74\x61\x72\x20\x20\x00", sizeof(magic)) == 0 ||
+	        memcmp(magic, "\x00\x00\x00\x00\x00\x00\x00\x00", sizeof(magic)) == 0);
 }
 
 static XArchiveType xa_tar_get_compressor_type (XArchive *archive)
@@ -53,6 +53,12 @@ static XArchiveType xa_tar_get_compressor_type (XArchive *archive)
 		case XARCHIVETYPE_TAR_GZ:
 			return XARCHIVETYPE_GZIP;
 
+		case XARCHIVETYPE_TAR_LZ:
+			return XARCHIVETYPE_LZIP;
+
+		case XARCHIVETYPE_TAR_LZ4:
+			return XARCHIVETYPE_LZ4;
+
 		case XARCHIVETYPE_TAR_LZMA:
 			return XARCHIVETYPE_LZMA;
 
@@ -62,6 +68,9 @@ static XArchiveType xa_tar_get_compressor_type (XArchive *archive)
 		case XARCHIVETYPE_TAR_XZ:
 			return XARCHIVETYPE_XZ;
 
+		case XARCHIVETYPE_TAR_Z:
+			return XARCHIVETYPE_COMPRESS;
+
 		default:
 			return XARCHIVETYPE_TAR;
 	}
@@ -79,27 +88,34 @@ void xa_tar_ask (XArchive *archive)
 	archive->can_update[0] = TRUE;
 	archive->can_update[1] = archiver[xa_tar_get_compressor_type(archive)].is_compressor;
 	archive->can_move = archiver[xa_tar_get_compressor_type(archive)].is_compressor;
+
+	/* this is solely for display with the archive properties */
+	if (!archiver[archive->type].program[0])
+	{
+		archiver[archive->type].program[0] = g_strdup(archiver[xa_tar_get_compressor_type(archive)].program[0]);
+		archiver[archive->type].program[1] = g_strdup(archiver[XARCHIVETYPE_TAR].program[0]);
+	}
 }
 
 static void xa_tar_parse_output (gchar *line, XArchive *archive)
 {
+	XEntry *entry;
 	gchar *filename;
 	gpointer item[6];
 	gint n = 0, a = 0 ,linesize = 0;
 
 	linesize = strlen(line);
-	archive->files++;
 
 	/* Permissions */
 	line[10] = '\0';
-	item[1] = line;
+	item[4] = line;
 
 	/* Owner */
 	for(n=13; n < linesize; ++n)
 		if(line[n] == ' ')
 			break;
 	line[n] = '\0';
-	item[2] = line+11;
+	item[5] = line+11;
 
 	/* Size */
 	for(++n; n < linesize; ++n)
@@ -112,8 +128,7 @@ static void xa_tar_parse_output (gchar *line, XArchive *archive)
 			break;
 
 	line[n] = '\0';
-	item[3] = line + a;
-	archive->files_size += g_ascii_strtoull(item[3],NULL,0);
+	item[1] = line + a;
 	a = ++n;
 
 	/* Date */
@@ -122,7 +137,7 @@ static void xa_tar_parse_output (gchar *line, XArchive *archive)
 			break;
 
 	line[n] = '\0';
-	item[4] = line + a;
+	item[2] = line + a;
 
 	/* Time */
 	a = ++n;
@@ -131,7 +146,7 @@ static void xa_tar_parse_output (gchar *line, XArchive *archive)
 			break;
 
 	line[n] = '\0';
-	item[5] = line + a;
+	item[3] = line + a;
 	n++;
 	line[linesize-1] = '\0';
 
@@ -158,14 +173,24 @@ static void xa_tar_parse_output (gchar *line, XArchive *archive)
 	}
 	else
 		filename = g_strdup(line + n);
-	xa_set_archive_entries_for_each_row (archive,filename,item);
+
+	entry = xa_set_archive_entries_for_each_row(archive, filename, item);
+
+	if (entry)
+	{
+		if (!entry->is_dir)
+			archive->files++;
+
+		archive->files_size += g_ascii_strtoull(item[1], NULL, 0);
+	}
+
 	g_free(filename);
 }
 
 void xa_tar_list (XArchive *archive)
 {
-	const GType types[] = {GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_UINT64, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER};
-	const gchar *titles[] = {_("Points to"), _("Permissions"), _("Owner/Group"), _("Size"), _("Date"), _("Time")};
+	const GType types[] = {GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_UINT64, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER};
+	const gchar *titles[] = {_("Points to"), _("Original Size"), _("Date"), _("Time"), _("Permissions"), _("Owner/Group")};
 	gchar *command;
 	guint i;
 
@@ -181,6 +206,7 @@ void xa_tar_list (XArchive *archive)
 	g_free(command);
 
 	archive->columns = 9;
+	archive->size_column = 3;
 	archive->column_types = g_malloc0(sizeof(types));
 
 	for (i = 0; i < archive->columns; i++)
@@ -231,49 +257,7 @@ gboolean xa_tar_extract (XArchive *archive, GSList *file_list)
 	/* collect all files that have been extracted to move them without full path */
 	if (result && !archive->do_full_path)
 	{
-		size_t offset;
-		GSList *stack;
-		GString *all_files = g_string_new("");
-
-		offset = strlen(extract_to) + 1;
-		stack = g_slist_append(NULL, g_strdup(extract_to));
-
-		while (stack)
-		{
-			gchar *file;
-
-			file = stack->data;
-			stack = g_slist_delete_link(stack, stack);
-
-			if (g_file_test(file, G_FILE_TEST_IS_DIR))
-			{
-				GDir *dir;
-				const gchar *name;
-
-				dir = g_dir_open(file, 0, NULL);
-
-				if (dir)
-				{
-					while ((name = g_dir_read_name(dir)))
-						stack = g_slist_prepend(stack, g_strconcat(file, "/", name, NULL));
-
-					g_dir_close(dir);
-				}
-			}
-			else
-			{
-				gchar *quoted = g_shell_quote(file + offset);
-
-				all_files = g_string_append_c(all_files, ' ');
-				all_files = g_string_append(all_files, quoted);
-
-				g_free(quoted);
-			}
-
-			g_free(file);
-		}
-
-		g_slist_free(stack);
+		GString *all_files = xa_collect_files_in_dir(extract_to);
 
 		archive->child_dir = g_strdup(extract_to);
 		command = g_strconcat("mv",
