@@ -42,6 +42,7 @@
 #endif
 
 #define	XDS_FILENAME "xds.txt"
+#define bswap(word) (((word << 8) & 0xff00) | ((word >> 8) & 0x00ff))
 
 gchar *current_open_directory;
 
@@ -353,11 +354,11 @@ static void xa_activate_link (GtkAboutDialog *about, const gchar *link, gpointer
 			g_free (browser_path);
 			return;
 		}
-		xa_launch_external_program(browser_path,(gchar *)link);
+		xa_launch_external_program(browser_path, link);
 		g_free(browser_path);
 	}
 	else
-		xa_launch_external_program(xdg_open, (gchar*) link);
+		xa_launch_external_program(xdg_open, link);
 }
 
 static void xa_rename_cell_edited_canceled (GtkCellRenderer *renderer, gpointer data)
@@ -433,12 +434,14 @@ done:
 
 static const gchar *xa_get_archive_format (XArchive *archive)
 {
-	GSList *list = archiver[archive->type].type;
+	gint pos;
 
-	if (archive->type == XARCHIVETYPE_RAR && archive->version == 5)
-		list = list->next;
+	pos = g_slist_index(archiver[archive->type].version, GUINT_TO_POINTER(archive->version));
 
-	return list->data;
+	if (pos >= 0)
+		return g_slist_nth_data(archiver[archive->type].version, pos + 1);
+	else
+		return archiver[archive->type].type->data;
 }
 
 static gchar *xa_get_statusbar_message (guint64 total_size, gint n_elem, gint dirs, gboolean selection)
@@ -453,7 +456,11 @@ static gchar *xa_get_statusbar_message (guint64 total_size, gint n_elem, gint di
 	if (dirs)
 	{
 		if (n_elem)
-			info = g_strdup_printf(ngettext ("%d file and %d dir %s (%s)","%d files and %d dirs %s (%s)",n_elem),n_elem,dirs,text,measure);
+		{
+			gchar *format = g_strconcat(ngettext("%d file", "%d files", n_elem), " ", _("and"), " ", ngettext("%d dir %s (%s)", "%d dirs %s (%s)", dirs), NULL);
+			info = g_strdup_printf(format, n_elem, dirs, text, measure);
+			g_free(format);
+		}
 		else
 			info = g_strdup_printf(ngettext ("%d dir %s (%s)","%d dirs %s (%s)",dirs),dirs,text,measure);
 	}
@@ -477,7 +484,6 @@ static void xa_determine_program_to_run (gchar *file)
 
 		basename = g_path_get_basename(file);
 		type = xa_get_stock_mime_icon(basename);
-		g_free(basename);
 
 		if (strcmp(type, "text-html") == 0)
 		{
@@ -489,11 +495,18 @@ static void xa_determine_program_to_run (gchar *file)
 		}
 		else if (strcmp(type, "image-x-generic") == 0)
 			program = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(prefs_window->combo_prefered_viewer));
+		else if (strcmp(type, "package-x-generic") == 0)
+			program = gtk_combo_box_text_get_active_text(GTK_COMBO_BOX_TEXT(prefs_window->combo_prefered_archiver));
 		else
 		{
-			xa_show_message_dialog(GTK_WINDOW(xa_main_window), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("This file type is not supported!"), _("Please install xdg-utils."));
+			gchar *basename_utf8 = g_filename_display_name(basename);
+			xa_create_open_with_dialog(basename_utf8, g_shell_quote(file), 1);
+			g_free(basename_utf8);
+			g_free(basename);
 			return;
 		}
+
+		g_free(basename);
 	}
 
 	if (program && *program)
@@ -546,7 +559,8 @@ static void xa_load_comment_window_from_file (GtkButton *button, gpointer data)
 	gtk_widget_destroy (file);
 	if (path != NULL)
 	{
-		response = g_file_get_contents(path,&content,NULL,&error);
+		response = g_file_get_contents(path, &content, &bytes, &error);
+
 		if (response == FALSE)
 		{
 			gchar *msg = g_strdup_printf (_("Can't open file %s:"),path);
@@ -558,8 +572,15 @@ static void xa_load_comment_window_from_file (GtkButton *button, gpointer data)
 			return;
 		}
 		g_free(path);
-		utf8_data = g_locale_to_utf8 (content,-1,NULL,&bytes,NULL);
-		g_free(content);
+
+		if (g_utf8_validate(content, -1, NULL))
+			utf8_data = content;
+		else
+		{
+			utf8_data = g_locale_to_utf8(content, -1, NULL, &bytes, NULL);
+			g_free(content);
+		}
+
 		textmark = gtk_text_buffer_get_insert(buf);
 		gtk_text_buffer_get_iter_at_mark(buf,&iter,textmark);
 		gtk_text_buffer_insert_with_tags_by_name (buf,&iter,utf8_data,bytes,"font",NULL);
@@ -651,14 +672,14 @@ static XAClipboard *xa_clipboard_data_new ()
 	return data;
 }
 
-static XAClipboard *xa_get_paste_data_from_clipboard_selection (const char *data)
+static XAClipboard *xa_get_paste_data_from_clipboard_selection (const guchar *data)
 {
 	gchar **uris;
 	gint i;
 	XAClipboard *clipboard_data;
 
 	clipboard_data = xa_clipboard_data_new();
-	uris = g_strsplit (data,"\r\n",-1);
+	uris = g_strsplit((const gchar *) data,"\r\n",-1);
 	clipboard_data->mode = (strcmp(uris[0], "copy") == 0 ? XA_CLIPBOARD_COPY : XA_CLIPBOARD_CUT);
 	sscanf(uris[1], "%p", &clipboard_data->target);
 	for (i = 2; uris[i]; i++)
@@ -744,7 +765,10 @@ void xa_child_processed (XAChildProcess process, gboolean success, XArchive *arc
 		archive->child_pid = 0;
 
 		if (xa_main_window)
+		{
 			gtk_widget_set_sensitive(Stop_button, FALSE);
+			xa_set_button_state(1, 1, 1, 1, archive->can_test, 1, archive->can_add, archive->can_extract, archive->can_sfx, archive->has_comment, archive->output, archive->has_password);
+		}
 	}
 
 	if (--archive->child_ref == 0)
@@ -765,7 +789,6 @@ void xa_child_processed (XAChildProcess process, gboolean success, XArchive *arc
 					gtk_widget_grab_focus(archive->treeview);
 				}
 
-				xa_set_button_state(1, 1, 1, 1, archive->can_test, 1, archive->can_add, archive->can_extract, archive->can_sfx, archive->has_comment, archive->output, archive->has_password);
 				xa_set_statusbar_message_for_displayed_rows(archive);
 
 				if (archive->status == XARCHIVESTATUS_LIST && archive->has_comment && gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_window->check_show_comment)))
@@ -785,16 +808,14 @@ void xa_child_processed (XAChildProcess process, gboolean success, XArchive *arc
 		}
 		else
 		{
-			if (xa_main_window)
-			{
-				if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_window->store_output)))
-					xa_show_message_dialog(GTK_WINDOW(xa_main_window), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("An error occurred!"), _("Please check the 'Store archiver output' option to see it."));
-
-				gtk_label_set_text(GTK_LABEL(total_label), _("An error occurred!"));
-			}
+			if (xa_main_window && !gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(prefs_window->store_output)))
+				xa_show_message_dialog(GTK_WINDOW(xa_main_window), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("An error occurred!"), _("Please check the 'Store archiver output' option to see it."));
 			else
 				xa_show_archive_output(NULL, archive);
 
+			if (xa_main_window)
+				gtk_label_set_text(GTK_LABEL(total_label), _("An error occurred!"));
+
 			/* in case the user has supplied a wrong password, reset it so they can try again */
 			if (archive->password && (archive->status == XARCHIVESTATUS_TEST || archive->status == XARCHIVESTATUS_SFX))
 			{
@@ -930,9 +951,6 @@ void xa_new_archive (GtkMenuItem *menuitem,gpointer user_data)
 	xa_set_button_state(1, 1, 1, 1, archive[current_page]->can_test, 1, archive[current_page]->can_add, archive[current_page]->can_extract, archive[current_page]->can_sfx, archive[current_page]->has_comment, archive[current_page]->output, archive[current_page]->has_password);
     xa_disable_delete_buttons(FALSE);
 
-    archive[current_page]->password = NULL;
-    archive[current_page]->files_size = 0;
-    archive[current_page]->files = 0;
 	xa_set_window_title(xa_main_window, archive[current_page]->path[0]);
 	gtk_label_set_text(GTK_LABEL(total_label),"");
 }
@@ -994,7 +1012,7 @@ void xa_open_archive (GtkMenuItem *menuitem,gpointer data)
 	gchar *utf8_path,*msg;
 	gint current_page;
 	gint x;
-	XArchiveType type;
+	ArchiveType xa;
 
 	path = (gchar *)data;
 	if (path == NULL)
@@ -1017,13 +1035,13 @@ void xa_open_archive (GtkMenuItem *menuitem,gpointer data)
 			return;
 		}
 	}
-	type = xa_detect_archive_type (path);
+	xa = xa_detect_archive_type(path);
 
-	if (type == XARCHIVETYPE_UNKNOWN || type == XARCHIVETYPE_NOT_FOUND)
+	if (xa.type == XARCHIVETYPE_UNKNOWN || xa.type == XARCHIVETYPE_NOT_FOUND)
 	{
 		utf8_path = g_filename_to_utf8 (path,-1,NULL,NULL,NULL);
 		msg = g_strdup_printf (_("Can't open file \"%s\":"),utf8_path);
-		if (type == XARCHIVETYPE_UNKNOWN)
+		if (xa.type == XARCHIVETYPE_UNKNOWN)
 			xa_show_message_dialog (GTK_WINDOW (xa_main_window),GTK_DIALOG_MODAL,GTK_MESSAGE_ERROR,GTK_BUTTONS_OK,msg,_("Archive format is not recognized!"));
 		else
 			xa_show_message_dialog (GTK_WINDOW (xa_main_window),GTK_DIALOG_MODAL,GTK_MESSAGE_ERROR,GTK_BUTTONS_OK,msg,g_strerror(errno));
@@ -1034,7 +1052,7 @@ void xa_open_archive (GtkMenuItem *menuitem,gpointer data)
 		return;
 	}
 
-	if (!list[type])
+	if (!list[xa.type])
 	{
 		xa_show_message_dialog(GTK_WINDOW(xa_main_window), GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_OK, _("Sorry, this archive format is not supported:"), _("The proper archiver is not installed!"));
 		g_free(path);
@@ -1047,7 +1065,7 @@ void xa_open_archive (GtkMenuItem *menuitem,gpointer data)
 		g_free (path);
 		return;
 	}
-	archive[current_page] = xa_init_archive_structure(type);
+	archive[current_page] = xa_init_archive_structure(xa);
 	if (archive[current_page] == NULL)
 	{
 		xa_show_message_dialog (GTK_WINDOW (xa_main_window),GTK_DIALOG_MODAL,GTK_MESSAGE_ERROR,GTK_BUTTONS_OK,_("Can't allocate memory for the archive structure:"),_("Operation aborted!"));
@@ -1055,9 +1073,9 @@ void xa_open_archive (GtkMenuItem *menuitem,gpointer data)
 		return;
 	}
 	/* Detect archive comment,rar one is detected in rar.c */
-	if (type == XARCHIVETYPE_ZIP)
+	if (xa.type == XARCHIVETYPE_ZIP)
 		archive[current_page]->has_comment = xa_detect_archive_comment (XARCHIVETYPE_ZIP,path,archive[current_page]);
-	else if (type == XARCHIVETYPE_ARJ)
+	else if (xa.type == XARCHIVETYPE_ARJ)
 		archive[current_page]->has_comment = xa_detect_archive_comment (XARCHIVETYPE_ARJ,path,archive[current_page]);
 
 	if (g_path_is_absolute(path) == FALSE)
@@ -1402,6 +1420,7 @@ void xa_show_prefs_dialog (GtkMenuItem *menuitem,gpointer user_data)
 		prefs_window = xa_create_prefs_dialog();
 
 	gtk_widget_show_all (prefs_window->dialog1);
+	xa_prefs_iconview_changed(GTK_ICON_VIEW(prefs_window->iconview), prefs_window);
 	response = gtk_dialog_run (GTK_DIALOG(prefs_window->dialog1));
 	gtk_widget_hide (prefs_window->dialog1);
 
@@ -1701,53 +1720,90 @@ void xa_about (GtkMenuItem *menuitem,gpointer user_data)
 	gtk_widget_hide (about);
 }
 
-XArchiveType xa_detect_archive_type (gchar *filename)
+ArchiveType xa_detect_archive_type (const gchar *filename)
 {
 	FILE *file;
 	unsigned char magic[14];
-	int type = XARCHIVETYPE_UNKNOWN;
+	unsigned short *short_magic = (unsigned short *) magic;
+	ArchiveType xa = {XARCHIVETYPE_UNKNOWN, 0};
 
 	file = fopen(filename, "r");
 
 	if (!file)
-		return XARCHIVETYPE_NOT_FOUND;
+	{
+		xa.type = XARCHIVETYPE_NOT_FOUND;
+		return xa;
+	}
 
 	memset(magic, 0, sizeof(magic));
 	fread(magic, 1, sizeof(magic), file);
 
 	if (memcmp(magic, "\x37\x7a\xbc\xaf\x27\x1c", 6) == 0)
-		type = XARCHIVETYPE_7ZIP;
+		xa.type = XARCHIVETYPE_7ZIP;
 	else if (memcmp(magic, "\x60\xea", 2) == 0)
-		type = XARCHIVETYPE_ARJ;
+		xa.type = XARCHIVETYPE_ARJ;
 	else if (memcmp(magic, "\x42\x5a\x68", 3) == 0)
-		type = XARCHIVETYPE_BZIP2;
+		xa.type = XARCHIVETYPE_BZIP2;
+	else if (memcmp(magic, "\x1f\x9d", 2) == 0)
+		xa.type = XARCHIVETYPE_COMPRESS;
+	else if (memcmp(magic, "070701", 6) == 0 ||
+	         memcmp(magic, "070702", 6) == 0 ||
+	         memcmp(magic, "070707", 6) == 0 ||
+	         *short_magic == 070707 ||
+	         *short_magic == bswap(070707))
+	{
+		xa.type = XARCHIVETYPE_CPIO;
+
+		if (*short_magic == bswap(070707))
+			xa.version = 'E';
+	}
 	else if (memcmp(magic, "!<arch>\ndebian", 14) == 0)
-		type = XARCHIVETYPE_DEB;
+		xa.type = XARCHIVETYPE_DEB;
 	else if (memcmp(magic, "\x1f\x8b", 2) == 0 ||
 	         memcmp(magic, "\x1f\x9d", 2) == 0)
-		type = XARCHIVETYPE_GZIP;
+		xa.type = XARCHIVETYPE_GZIP;
 	else if ((memcmp(magic + 2, "-lh", 3) == 0 && ((magic[5] >= '0' && magic[5] <= '7') || magic[5] == 'd') && magic[6] == '-') ||
 	         (memcmp(magic + 2, "-lz", 3) == 0 && (magic[5] == '4' || magic[5] == '5' || magic[5] == 's') && magic[6] == '-'))
-		type = XARCHIVETYPE_LHA;
+		xa.type = XARCHIVETYPE_LHA;
+	else if (memcmp(magic, "\x04\x22\x4d\x18", 4) == 0 ||
+	         memcmp(magic, "\x02\x21\x4c\x18", 4) == 0 ||
+	         (memcmp(magic + 1, "\x2a\x4d\x18", 3) == 0 && (magic[0] & 0xf0) == 0x50))
+		xa.type = XARCHIVETYPE_LZ4;
+	else if (memcmp(magic, "\x4c\x5a\x49\x50", 4) == 0)
+		xa.type = XARCHIVETYPE_LZIP;
 	else if (memcmp(magic, "\x5d\x00\x00\x80", 4) == 0)
-		type = XARCHIVETYPE_LZMA;
+		xa.type = XARCHIVETYPE_LZMA;
 	else if (memcmp(magic, "\211LZO", 4) == 0)
-		type = XARCHIVETYPE_LZOP;
+		xa.type = XARCHIVETYPE_LZOP;
 	else if (memcmp(magic, "\x52\x61\x72\x21\x1a\x07\x00", 7) == 0 ||
 	         memcmp(magic, "\x52\x61\x72\x21\x1a\x07\x01", 7) == 0)
-		type = XARCHIVETYPE_RAR;
+	{
+		xa.type = XARCHIVETYPE_RAR;
+
+		if (magic[6] == 1)
+			xa.version = 5;
+
+		/* a rar5 archive without rar v5 executable can't be opened */
+		if ((xa.version == 5) && !g_slist_find(archiver[xa.type].version, GUINT_TO_POINTER(xa.version)))
+			list[xa.type] = NULL;
+	}
 	else if (memcmp(magic, "\xed\xab\xee\xdb", 4) == 0)
-		type = XARCHIVETYPE_RPM;
+		xa.type = XARCHIVETYPE_RPM;
 	else if (isTar(file))
-		type = XARCHIVETYPE_TAR;
-	else if (memcmp(magic, "\xfd\x37\x7a\x58\x5a", 5) == 0)
-		type = XARCHIVETYPE_XZ;
+		xa.type = XARCHIVETYPE_TAR;
+	else if (memcmp(magic, "\xfd\x37\x7a\x58\x5a\x00", 6) == 0)
+		xa.type = XARCHIVETYPE_XZ;
 	else if (memcmp(magic, "\x50\x4b", 2) == 0)
-		type = XARCHIVETYPE_ZIP;
+	{
+		xa.type = XARCHIVETYPE_ZIP;
+
+		if (g_str_has_suffix(filename, ".jar"))
+			xa.version = 'j';
+	}
 
 	fclose(file);
 
-	return type;
+	return xa;
 }
 
 void xa_create_liststore (XArchive *archive, const gchar *titles[])
@@ -1881,6 +1937,12 @@ void xa_archive_properties (GtkMenuItem *menuitem,gpointer user_data)
     g_free ( utf8_string);
 	/* Type */
 	gtk_label_set_text(GTK_LABEL(type_data), xa_get_archive_format(archive[idx]));
+	/* archiver */
+	t = g_strconcat(archiver[archive[idx]->type].program[0], archiver[archive[idx]->type].program[1] ? "\n" : "", archiver[archive[idx]->type].program[1], NULL);
+	utf8_string = g_filename_display_name(t);
+	gtk_label_set_text(GTK_LABEL(archiver_data), utf8_string);
+	g_free(utf8_string);
+	g_free(t);
     /* Modified Date */
     strftime (date,64,"%c",localtime (&my_stat.st_mtime));
     t = g_locale_to_utf8 (date,-1,0,0,0);
@@ -1901,8 +1963,13 @@ void xa_archive_properties (GtkMenuItem *menuitem,gpointer user_data)
 		gtk_label_set_text(GTK_LABEL(comment_data),_("No"));
 
     /* Compression_ratio */
-    content_size = (double)archive[idx]->files_size / file_size;
-    t = g_strdup_printf ( "%.2f",content_size);
+    if (file_size)
+    {
+      content_size = (double) archive[idx]->files_size / file_size;
+      t = g_strdup_printf("%.2f:1", content_size);
+    }
+    else
+      t = g_strdup("-");
     gtk_label_set_text(GTK_LABEL(compression_data),t);
     g_free (t);
     /* Number of files */
@@ -1922,7 +1989,7 @@ void xa_set_statusbar_message_for_displayed_rows(XArchive *archive)
 	gchar *info = NULL;
 	GtkTreePath *path = NULL;
 	GtkTreeIter iter;
-	gint n_elem = 0,pos = 0,dirs = 0;
+	gint n_elem = 0, dirs = 0;
 	guint64 total_size = 0;
 	guint64 size = 0;
 	XEntry *entry = NULL;
@@ -1934,48 +2001,11 @@ void xa_set_statusbar_message_for_displayed_rows(XArchive *archive)
 		return;
 	}
 
-	switch (archive->type)
-	{
-		case XARCHIVETYPE_RPM:
-		pos = 3;
-		break;
-
-		case XARCHIVETYPE_BZIP2:
-		case XARCHIVETYPE_GZIP:
-		case XARCHIVETYPE_LZMA:
-		case XARCHIVETYPE_LZOP:
-		case XARCHIVETYPE_XZ:
-		case XARCHIVETYPE_RAR:
-		case XARCHIVETYPE_ARJ:
-		case XARCHIVETYPE_7ZIP:
-		pos = 2;
-		break;
-
-		case XARCHIVETYPE_DEB:
-		pos = 4;
-		break;
-
-		case XARCHIVETYPE_LHA:
-		case XARCHIVETYPE_TAR_GZ:
-		case XARCHIVETYPE_TAR_BZ2:
-		case XARCHIVETYPE_TAR_LZMA:
-		case XARCHIVETYPE_TAR_XZ:
-		case XARCHIVETYPE_TAR_LZOP:
-		case XARCHIVETYPE_TAR:
-		case XARCHIVETYPE_ZIP:
-		pos = 5;
-		break;
-
-		default:
-		break;
-	}
 	gtk_tree_path_free(path);
 	do
 	{
-		gtk_tree_model_get (archive->model,&iter,pos,&size,-1);
+		gtk_tree_model_get(archive->model, &iter, archive->size_column, &size, -1);
 		gtk_tree_model_get(archive->model, &iter, archive->columns - 1, &entry, -1);
-		if (entry == NULL)
-			return;
 		if (entry->is_dir)
 			dirs++;
 		else
@@ -1993,48 +2023,11 @@ void xa_row_selected (GtkTreeSelection *selection,XArchive *archive)
 	GList *list = NULL;
 	gchar *msg = NULL;
 	GtkTreeIter iter;
-	GtkTreeModel *model;
-	gint selected = 0,pos = 0,dirs = 0;
+	gint selected, dirs = 0;
 	guint64 total_size = 0;
 	guint64 size = 0;
 	XEntry *entry;
 
-	switch (archive->type)
-	{
-		case XARCHIVETYPE_RPM:
-		pos = 3;
-		break;
-
-		case XARCHIVETYPE_BZIP2:
-		case XARCHIVETYPE_GZIP:
-		case XARCHIVETYPE_LZMA:
-		case XARCHIVETYPE_LZOP:
-		case XARCHIVETYPE_XZ:
-		case XARCHIVETYPE_RAR:
-		case XARCHIVETYPE_ARJ:
-		case XARCHIVETYPE_7ZIP:
-		pos = 2;
-		break;
-
-		case XARCHIVETYPE_DEB:
-		pos = 4;
-		break;
-
-		case XARCHIVETYPE_LHA:
-		case XARCHIVETYPE_TAR_GZ:
-		case XARCHIVETYPE_TAR_BZ2:
-		case XARCHIVETYPE_TAR_LZMA:
-		case XARCHIVETYPE_TAR_XZ:
-		case XARCHIVETYPE_TAR_LZOP:
-		case XARCHIVETYPE_TAR:
-		case XARCHIVETYPE_ZIP:
-		pos = 5;
-		break;
-
-		default:
-		break;
-	}
-	model = gtk_tree_view_get_model(GTK_TREE_VIEW(archive->treeview));
 	selected = gtk_tree_selection_count_selected_rows (selection);
 
 	if (selected == 0)
@@ -2043,24 +2036,18 @@ void xa_row_selected (GtkTreeSelection *selection,XArchive *archive)
 		gtk_widget_hide(selected_frame);
 		return;
 	}
-	else
-	{
-		gtk_widget_show(selected_frame);
-		gtk_widget_set_sensitive(deselect_all,TRUE);
-	}
 
+	gtk_widget_show(selected_frame);
+	gtk_widget_set_sensitive(deselect_all, TRUE);
 	gtk_widget_set_sensitive(delete_menu, archive->can_delete);
-	gtk_widget_set_sensitive(rename_menu, can_rename(archive));
-
-	if (selected > 1)
-		gtk_widget_set_sensitive (rename_menu,FALSE);
+	gtk_widget_set_sensitive(rename_menu, selected == 1 ? can_rename(archive) : FALSE);
 
 	selected = 0;
 	list = gtk_tree_selection_get_selected_rows(selection,NULL);
 	while (list)
 	{
-		gtk_tree_model_get_iter(model,&iter,list->data);
-		gtk_tree_model_get (model,&iter,pos,&size,-1);
+		gtk_tree_model_get_iter(archive->model, &iter, list->data);
+		gtk_tree_model_get(archive->model, &iter, archive->size_column, &size, -1);
 		gtk_tree_model_get(archive->model, &iter, archive->columns - 1, &entry, -1);
 		if (entry->is_dir)
 			dirs++;
@@ -2194,7 +2181,8 @@ void on_drag_data_received (GtkWidget *widget,GdkDragContext *context,int x,int
 	GSList *list = NULL;
 	gboolean one_file;
 	unsigned int len = 0;
-	gint current_page, idx, type;
+	gint current_page, idx;
+	ArchiveType xa;
 
 	current_page = gtk_notebook_get_current_page(notebook);
 	idx = xa_find_archive_index (current_page);
@@ -2216,9 +2204,9 @@ void on_drag_data_received (GtkWidget *widget,GdkDragContext *context,int x,int
 		if (filename == NULL)
 			return;
 
-		type = xa_detect_archive_type(filename);
+		xa = xa_detect_archive_type(filename);
 
-		if (type != XARCHIVETYPE_UNKNOWN && type != XARCHIVETYPE_NOT_FOUND)
+		if (xa.type != XARCHIVETYPE_UNKNOWN && xa.type != XARCHIVETYPE_NOT_FOUND)
 		{
 			xa_open_archive(NULL,filename);
 			g_strfreev(array);
@@ -2307,7 +2295,7 @@ void xa_set_environment (gpointer user_data)
 	g_setenv("DISPLAY", (const gchar *) user_data, TRUE);
 }
 
-gboolean xa_launch_external_program(gchar *program,gchar *arg)
+gboolean xa_launch_external_program (const gchar *program, const gchar *arg)
 {
 	GtkWidget *message;
 	GError *error = NULL;
@@ -2441,7 +2429,14 @@ void xa_show_archive_comment (GtkMenuItem *menuitem,gpointer user_data)
 
 	if (archive[idx]->comment)
 	{
-		utf8_line = g_locale_to_utf8 (archive[idx]->comment->str,-1,NULL,&len,NULL);
+		if (g_utf8_validate(archive[idx]->comment->str, -1, NULL))
+		{
+			utf8_line = g_strdup(archive[idx]->comment->str);
+			len = -1;
+		}
+		else
+			utf8_line = g_locale_to_utf8(archive[idx]->comment->str, -1, NULL, &len, NULL);
+
 		gtk_text_buffer_insert_with_tags_by_name (textbuffer,&iter,utf8_line,len,"font",NULL);
 		g_free(utf8_line);
 	}
@@ -2469,7 +2464,7 @@ void xa_location_entry_activated (GtkEntry *entry,gpointer user_data)
 	}
 
 	pathname_local = g_filename_from_utf8(gtk_entry_get_text(GTK_ENTRY(location_entry)), -1, NULL, NULL, NULL);
-	new_entry = xa_find_entry_from_path(archive[idx]->root_entry, pathname_local);
+	new_entry = xa_find_entry_from_dirpath(archive[idx], pathname_local);
 	g_free(pathname_local);
 	if (new_entry == NULL)
 	{
@@ -2483,7 +2478,7 @@ void xa_location_entry_activated (GtkEntry *entry,gpointer user_data)
 	}
 
 	if (archive[idx]->location_path)
-		prev_entry = xa_find_entry_from_path(archive[idx]->root_entry, archive[idx]->location_path);
+		prev_entry = xa_find_entry_from_dirpath(archive[idx], archive[idx]->location_path);
 
 	if (prev_entry != NULL)
 		archive[idx]->back = g_slist_prepend(archive[idx]->back,prev_entry);
@@ -2494,11 +2489,11 @@ void xa_location_entry_activated (GtkEntry *entry,gpointer user_data)
 	xa_update_window_with_archive_entries(archive[idx],new_entry);
 }
 
-int xa_mouse_button_event(GtkWidget *widget,GdkEventButton *event,XArchive *archive)
+gboolean xa_mouse_button_event (GtkWidget *widget, GdkEventButton *event, XArchive *archive)
 {
 	XEntry *entry;
 	GtkTreePath *path;
-	GtkTreeIter  iter;
+	GtkTreeIter iter;
 	GtkTreeSelection *selection;
 	gint selected;
 	GtkClipboard *clipboard;
@@ -2506,53 +2501,66 @@ int xa_mouse_button_event(GtkWidget *widget,GdkEventButton *event,XArchive *arch
 	XAClipboard *paste_data;
 	gboolean pasteable = FALSE;
 
-	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (archive->treeview));
+	selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(archive->treeview));
 	selected = gtk_tree_selection_count_selected_rows(selection);
-	gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (archive->treeview),event->x,event->y,&path,NULL,NULL,NULL);
+	gtk_tree_view_get_path_at_pos(GTK_TREE_VIEW(archive->treeview), event->x, event->y, &path, NULL, NULL, NULL);
+
 	if (path == NULL)
 		return FALSE;
-	if ((event->type == GDK_BUTTON_PRESS) && (event->button == 3))
+
+	if (event->type == GDK_BUTTON_PRESS && (event->button == 2 || event->button == 3))
 	{
-		gtk_tree_model_get_iter (GTK_TREE_MODEL (archive->liststore),&iter,path);
-		gtk_tree_path_free (path);
+		gtk_tree_model_get_iter(GTK_TREE_MODEL(archive->liststore), &iter, path);
 		gtk_tree_model_get(archive->model, &iter, archive->columns - 1, &entry, -1);
-		if (! gtk_tree_selection_iter_is_selected (selection,&iter))
+
+		if (!gtk_tree_selection_iter_is_selected(selection, &iter))
 		{
-			gtk_tree_selection_unselect_all (selection);
-			gtk_tree_selection_select_iter (selection,&iter);
+			gtk_tree_selection_unselect_all(selection);
+			gtk_tree_selection_select_iter(selection, &iter);
+			selected = 1;
 		}
 
-		if (selected > 1 || entry->is_dir)
+		if (event->button == 2 && selected <= 1)
+			xa_treeview_row_activated(GTK_TREE_VIEW(archive->treeview), path, NULL, archive);
+		else if (event->button == 3)
 		{
-			gtk_widget_set_sensitive(open_popupmenu, FALSE);
-			gtk_widget_set_sensitive(view, FALSE);
-			gtk_widget_set_sensitive(rrename, FALSE);
-		}
-		else
-		{
-			gtk_widget_set_sensitive(open_popupmenu, archive->can_extract);
-			gtk_widget_set_sensitive(view, archive->can_extract);
-			gtk_widget_set_sensitive(rrename, can_rename(archive));
+			if (selected > 1 || entry->is_dir)
+			{
+				gtk_widget_set_sensitive(open_popupmenu, FALSE);
+				gtk_widget_set_sensitive(view, FALSE);
+				gtk_widget_set_sensitive(rrename, FALSE);
+			}
+			else
+			{
+				gtk_widget_set_sensitive(open_popupmenu, archive->can_extract);
+				gtk_widget_set_sensitive(view, archive->can_extract);
+				gtk_widget_set_sensitive(rrename, can_rename(archive));
+			}
+
+			clipboard = gtk_clipboard_get(XA_CLIPBOARD);
+			clipboard_selection = gtk_clipboard_wait_for_contents(clipboard, XA_INFO_LIST);
+
+			if (clipboard_selection != NULL)
+			{
+				paste_data = xa_get_paste_data_from_clipboard_selection(gtk_selection_data_get_data(clipboard_selection));
+				gtk_selection_data_free(clipboard_selection);
+
+				pasteable = (strcmp(archive->path[1], paste_data->target->path[1]) != 0);
+			}
+
+			gtk_widget_set_sensitive(eextract, archive->can_extract);
+			gtk_widget_set_sensitive(cut, archive->can_extract && archive->can_delete);
+			gtk_widget_set_sensitive(copy, archive->can_extract);
+			gtk_widget_set_sensitive(paste, pasteable && archive->can_add);
+			gtk_widget_set_sensitive(ddelete, archive->can_delete);
+			gtk_menu_popup(GTK_MENU(xa_popup_menu), NULL, NULL, NULL, xa_main_window, event->button, event->time);
 		}
 
-		clipboard = gtk_clipboard_get(XA_CLIPBOARD);
-		clipboard_selection = gtk_clipboard_wait_for_contents(clipboard,XA_INFO_LIST);
-		if (clipboard_selection != NULL)
-		{
-			paste_data = xa_get_paste_data_from_clipboard_selection((char *) gtk_selection_data_get_data(clipboard_selection));
-			gtk_selection_data_free (clipboard_selection);
-			if (strcmp(archive->path[1], paste_data->target->path[1]) != 0)
-				pasteable = TRUE;
-		}
-
-		gtk_widget_set_sensitive(eextract, archive->can_extract);
-		gtk_widget_set_sensitive(cut, archive->can_extract && archive->can_delete);
-		gtk_widget_set_sensitive(copy, archive->can_extract);
-		gtk_widget_set_sensitive(paste, pasteable && archive->can_add);
-		gtk_widget_set_sensitive(ddelete, archive->can_delete);
-		gtk_menu_popup (GTK_MENU (xa_popup_menu),NULL,NULL,NULL,xa_main_window,event->button,event->time);
 		return TRUE;
 	}
+
+	gtk_tree_path_free(path);
+
 	return FALSE;
 }
 
@@ -2590,7 +2598,7 @@ void xa_clipboard_paste(GtkMenuItem* item,gpointer data)
 	selection = gtk_clipboard_wait_for_contents(clipboard,XA_INFO_LIST);
 	if (selection == NULL)
 		return;
-	paste_data = xa_get_paste_data_from_clipboard_selection((char *) gtk_selection_data_get_data(selection));
+	paste_data = xa_get_paste_data_from_clipboard_selection(gtk_selection_data_get_data(selection));
 	gtk_selection_data_free (selection);
 
 	/* Let's add the already extracted files in the tmp dir to the current archive dir */
@@ -2713,6 +2721,7 @@ void xa_open_with_from_popupmenu(GtkMenuItem *item,gpointer data)
 	}
 	while (list_of_files);
 	xa_create_open_with_dialog(entry->filename,names->str,nr);
+	g_string_free(names, FALSE);
 	g_slist_foreach(list_of_files,(GFunc)g_free,NULL);
 	g_slist_free(list_of_files);
 }
@@ -2781,7 +2790,7 @@ void xa_treeview_row_activated(GtkTreeView *tree_view,GtkTreePath *path,GtkTreeV
 	if (entry->is_dir)
 	{
 		if (archive->location_path)
-			archive->back = g_slist_prepend(archive->back, xa_find_entry_from_path(archive->root_entry, archive->location_path));
+			archive->back = g_slist_prepend(archive->back, xa_find_entry_from_dirpath(archive, archive->location_path));
 		/* Put NULL so to display the root entry */
 		else
 			archive->back = g_slist_prepend(archive->back,NULL);
@@ -2827,7 +2836,7 @@ void xa_update_window_with_archive_entries (XArchive *archive,XEntry *entry)
 	gchar *entry_utf8;
 
 	if (archive->status == XARCHIVESTATUS_RELOAD && archive->location_path != NULL)
-		entry = xa_find_entry_from_path(archive->root_entry, archive->location_path);
+		entry = xa_find_entry_from_dirpath(archive, archive->location_path);
 	else
 		archive->current_entry = entry;
 
