@@ -44,14 +44,14 @@ void xa_convert_sfx(GtkMenuItem *, gpointer);
 void xa_create_liststore(XArchive *, const gchar *[]);
 void xa_delete_archive(GtkMenuItem *, gpointer);
 void xa_deselect_all(GtkMenuItem *, gpointer);
-XArchiveType xa_detect_archive_type(gchar *);
+ArchiveType xa_detect_archive_type(const gchar *);
 void xa_enter_password(GtkMenuItem *, gpointer);
 void xa_extract_archive(GtkMenuItem *, gpointer);
-gboolean xa_launch_external_program(gchar *, gchar *);
+gboolean xa_launch_external_program(const gchar *, const gchar *);
 void xa_list_archive(GtkMenuItem *, gpointer);
 void xa_location_entry_activated(GtkEntry *, gpointer);
 GtkWidget *xa_main_window_find_image(gchar *, GtkIconSize);
-int xa_mouse_button_event(GtkWidget *, GdkEventButton *, XArchive *);
+gboolean xa_mouse_button_event(GtkWidget *, GdkEventButton *, XArchive *);
 void xa_new_archive(GtkMenuItem *, gpointer);
 void xa_open_archive(GtkMenuItem *, gpointer);
 void xa_open_with_from_popupmenu(GtkMenuItem *, gpointer);
